@@ -33,12 +33,14 @@
  * <example>
  * 	<title>Authenticating and Creating a New Album</title>
  * 	<programlisting>
+ *	GDataClientLoginAuthorizer *authorizer;
  *	GDataPicasaWebService *service;
  *	GDataPicasaWebAlbum *album, *inserted_album;
  *
- *	/<!-- -->* Create a service object and authenticate with the PicasaWeb server *<!-- -->/
- *	service = gdata_picasaweb_service_new ("companyName-applicationName-versionID");
- *	gdata_service_authenticate (GDATA_SERVICE (service), username, password, NULL, NULL);
+ *	/<!-- -->* Create a service object and authorize against the PicasaWeb service *<!-- -->/
+ *	authorizer = gdata_client_login_authorizer_new ("companyName-applicationName-versionID", GDATA_TYPE_PICASAWEB_SERVICE);
+ *	gdata_client_login_authorizer_authenticate (authorizer, username, password, NULL, NULL);
+ *	service = gdata_picasaweb_service_new (GDATA_AUTHORIZER (authorizer));
  *
  *	/<!-- -->* Create a GDataPicasaWebAlbum entry for the new album, setting some information about it *<!-- -->/
  *	album = gdata_picasaweb_album_new (NULL);
@@ -52,6 +54,7 @@
  *	g_object_unref (album);
  *	g_object_unref (inserted_album);
  *	g_object_unref (service);
+ *	g_object_unref (authorizer);
  *	</programlisting>
  * </example>
  *
@@ -59,22 +62,46 @@
  * 	<title>Uploading a Photo or Video</title>
  * 	<programlisting>
  *	GDataPicasaWebFile *file_entry, *uploaded_file_entry;
+ *	GDataUploadStream *upload_stream;
  *	GFile *file_data;
+ *	GFileInfo *file_info;
+ *	GFileInputStream *file_stream;
  *
  *	/<!-- -->* Specify the GFile image on disk to upload *<!-- -->/
  *	file_data = g_file_new_for_path (path);
  *
+ *	/<!-- -->* Get the file information for the file being uploaded. If another data source was being used for the upload, it would have to
+ *	 * provide an appropriate slug and content type. Note that this is a blocking operation. *<!-- -->/
+ *	file_info = g_file_query_info (file_data, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME "," G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+ *	                               G_FILE_QUERY_INFO_NONE, NULL, NULL);
+ *
  *	/<!-- -->* Create a GDataPicasaWebFile entry for the image, setting a title and caption/summary *<!-- -->/
  *	file_entry = gdata_picasaweb_file_new (NULL);
  *	gdata_entry_set_title (GDATA_ENTRY (file_entry), "Black Cat");
  *	gdata_entry_set_summary (GDATA_ENTRY (file_entry), "Photo of the world's most beautiful cat.");
  *
- *	/<!-- -->* Upload the file to the server. Note that this is a blocking operation. *<!-- -->/
- *	uploaded_file_entry = gdata_picasaweb_service_upload_file (service, album, file_entry, file_data, NULL, NULL);
- *
+ *	/<!-- -->* Create an upload stream for the file. This is non-blocking. *<!-- -->/
+ *	upload_stream = gdata_picasaweb_service_upload_file (service, album, file_entry, g_file_info_get_display_name (file_info),
+ *	                                                     g_file_info_get_content_type (file_info), NULL, NULL);
+ *	g_object_unref (file_info);
  *	g_object_unref (file_entry);
- *	g_object_unref (uploaded_file_entry);
+ *
+ *	/<!-- -->* Prepare a file stream for the file to be uploaded. This is a blocking operation. *<!-- -->/
+ *	file_stream = g_file_read (file_data, NULL, NULL);
  *	g_object_unref (file_data);
+ *
+ *	/<!-- -->* Upload the file to the server. Note that this is a blocking operation. *<!-- -->/
+ *	g_output_stream_splice (G_OUTPUT_STREAM (upload_stream), G_INPUT_STREAM (file_stream),
+ *	                        G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET, NULL, NULL);
+ *
+ *	/<!-- -->* Parse the resulting updated entry. This is a non-blocking operation. *<!-- -->/
+ *	uploaded_file_entry = gdata_picasaweb_service_finish_file_upload (service, upload_stream, NULL);
+ *	g_object_unref (file_stream);
+ *	g_object_unref (upload_stream);
+ *
+ *	/<!-- -->* ... *<!-- -->/
+ *
+ *	g_object_unref (uploaded_file_entry);
  * 	</programlisting>
  * </example>
  *
@@ -94,14 +121,17 @@
 #include "gdata-upload-stream.h"
 #include "gdata-picasaweb-feed.h"
 
+static GList *get_authorization_domains (void);
+
+_GDATA_DEFINE_AUTHORIZATION_DOMAIN (picasaweb, "lh2", "http://picasaweb.google.com/data/")
 G_DEFINE_TYPE (GDataPicasaWebService, gdata_picasaweb_service, GDATA_TYPE_SERVICE)
 
 static void
 gdata_picasaweb_service_class_init (GDataPicasaWebServiceClass *klass)
 {
 	GDataServiceClass *service_class = GDATA_SERVICE_CLASS (klass);
-	service_class->service_name = "lh2";
 	service_class->feed_type = GDATA_TYPE_PICASAWEB_FEED;
+	service_class->get_authorization_domains = get_authorization_domains;
 }
 
 static void
@@ -110,23 +140,49 @@ gdata_picasaweb_service_init (GDataPicasaWebService *self)
 	/* Nothing to see here */
 }
 
+static GList *
+get_authorization_domains (void)
+{
+	return g_list_prepend (NULL, get_picasaweb_authorization_domain ());
+}
+
 /**
  * gdata_picasaweb_service_new:
- * @client_id: your application's client ID
+ * @authorizer: (allow-none): a #GDataAuthorizer to authorize the service's requests, or %NULL
  *
- * Creates a new #GDataPicasaWebService. The @client_id must be unique for your application, and as registered with Google.
- * The <ulink type="http" url="http://code.google.com/apis/accounts/docs/AuthForInstalledApps.html#Request">recommended
- * form</ulink> is "companyName-applicationName-versionID".
+ * Creates a new #GDataPicasaWebService using the given #GDataAuthorizer. If @authorizer is %NULL, all requests are made as an unauthenticated user.
  *
- * Return value: a new #GDataPicasaWebService, or %NULL
+ * Return value: a new #GDataPicasaWebService, or %NULL; unref with g_object_unref()
  *
- * Since: 0.4.0
- **/
+ * Since: 0.9.0
+ */
 GDataPicasaWebService *
-gdata_picasaweb_service_new (const gchar *client_id)
+gdata_picasaweb_service_new (GDataAuthorizer *authorizer)
 {
-	g_return_val_if_fail (client_id != NULL, NULL);
-	return g_object_new (GDATA_TYPE_PICASAWEB_SERVICE, "client-id", client_id, NULL);
+	g_return_val_if_fail (authorizer == NULL || GDATA_IS_AUTHORIZER (authorizer), NULL);
+
+	return g_object_new (GDATA_TYPE_PICASAWEB_SERVICE,
+	                     "authorizer", authorizer,
+	                     NULL);
+}
+
+/**
+ * gdata_picasaweb_service_get_primary_authorization_domain:
+ *
+ * The primary #GDataAuthorizationDomain for interacting with PicasaWeb. This will not normally need to be used, as it's used internally
+ * by the #GDataPicasaWebService methods. However, if using the plain #GDataService methods to implement custom queries or requests which libgdata
+ * does not support natively, then this domain may be needed to authorize the requests.
+ *
+ * The domain never changes, and is interned so that pointer comparison can be used to differentiate it from other authorization domains.
+ *
+ * Return value: (transfer none): the service's authorization domain
+ *
+ * Since: 0.9.0
+ */
+GDataAuthorizationDomain *
+gdata_picasaweb_service_get_primary_authorization_domain (void)
+{
+	return get_picasaweb_authorization_domain ();
 }
 
 /*
@@ -145,22 +201,24 @@ static gchar *
 create_uri (GDataPicasaWebService *self, const gchar *username, const gchar *type)
 {
 	if (username == NULL) {
-		/* Ensure we're authenticated first */
-		if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE)
+		/* Ensure we're authorized first */
+		if (gdata_authorizer_is_authorized_for_domain (gdata_service_get_authorizer (GDATA_SERVICE (self)),
+		                                               get_picasaweb_authorization_domain ()) == FALSE) {
 			return NULL;
+		}
 
 		/* Querying Picasa albums for the "default" user when logged in returns the albums for the authenticated user */
 		username = "default";
 	}
 
-	return _gdata_service_build_uri (TRUE, "http://picasaweb.google.com/data/%s/api/user/%s", type, username);
+	return _gdata_service_build_uri ("https://picasaweb.google.com/data/%s/api/user/%s", type, username);
 }
 
 /**
  * gdata_picasaweb_service_get_user:
  * @self: a #GDataPicasaWebService
  * @username: (allow-none): the username of the user whose information you wish to retrieve, or %NULL for the currently authenticated user.
- * @cancellable: optional #GCancellable object, or %NULL
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
  * @error: a #GError, or %NULL
  *
  * Queries the service to return the user specified by @username.
@@ -187,7 +245,7 @@ gdata_picasaweb_service_get_user (GDataPicasaWebService *self, const gchar *user
 		return NULL;
 	}
 
-	message = _gdata_service_query (GDATA_SERVICE (self), uri, NULL, cancellable, error);
+	message = _gdata_service_query (GDATA_SERVICE (self), get_picasaweb_authorization_domain (), uri, NULL, cancellable, error);
 	g_free (uri);
 
 	if (message == NULL)
@@ -205,8 +263,8 @@ gdata_picasaweb_service_get_user (GDataPicasaWebService *self, const gchar *user
  * @self: a #GDataPicasaWebService
  * @query: (allow-none): a #GDataQuery with the query parameters, or %NULL
  * @username: (allow-none): the username of the user whose albums you wish to retrieve, or %NULL
- * @cancellable: optional #GCancellable object, or %NULL
- * @progress_callback: (scope call): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
+ * @progress_callback: (allow-none) (scope call) (closure progress_user_data): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
  * @progress_user_data: (closure): data to pass to the @progress_callback function
  * @error: a #GError, or %NULL
  *
@@ -249,7 +307,7 @@ gdata_picasaweb_service_query_all_albums (GDataPicasaWebService *self, GDataQuer
 	}
 
 	/* Execute the query */
-	album_feed = gdata_service_query (GDATA_SERVICE (self), uri, query, GDATA_TYPE_PICASAWEB_ALBUM,
+	album_feed = gdata_service_query (GDATA_SERVICE (self), get_picasaweb_authorization_domain (), uri, query, GDATA_TYPE_PICASAWEB_ALBUM,
 	                                  cancellable, progress_callback, progress_user_data, error);
 	g_free (uri);
 
@@ -261,8 +319,8 @@ gdata_picasaweb_service_query_all_albums (GDataPicasaWebService *self, GDataQuer
  * @self: a #GDataPicasaWebService
  * @query: (allow-none): a #GDataQuery with the query parameters, or %NULL
  * @username: (allow-none): the username of the user whose albums you wish to retrieve, or %NULL
- * @cancellable: optional #GCancellable object, or %NULL
- * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
+ * @progress_callback: (allow-none) (closure progress_user_data): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
  * @progress_user_data: (closure): data to pass to the @progress_callback function
  * @callback: a #GAsyncReadyCallback to call when authentication is finished
  * @user_data: (closure): data to pass to the @callback function
@@ -305,18 +363,37 @@ gdata_picasaweb_service_query_all_albums_async (GDataPicasaWebService *self, GDa
 	}
 
 	/* Schedule the async query */
-	gdata_service_query_async (GDATA_SERVICE (self), uri, query, GDATA_TYPE_PICASAWEB_ALBUM, cancellable, progress_callback, progress_user_data,
-	                           callback, user_data);
+	gdata_service_query_async (GDATA_SERVICE (self), get_picasaweb_authorization_domain (), uri, query, GDATA_TYPE_PICASAWEB_ALBUM, cancellable,
+	                           progress_callback, progress_user_data, callback, user_data);
 	g_free (uri);
 }
 
+static const gchar *
+get_query_files_uri (GDataPicasaWebAlbum *album, GError **error)
+{
+	if (album != NULL) {
+		GDataLink *_link = gdata_entry_look_up_link (GDATA_ENTRY (album), "http://schemas.google.com/g/2005#feed");
+		if (_link == NULL) {
+			/* Error */
+			g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
+			                     _("The album did not have a feed link."));
+			return NULL;
+		}
+
+		return gdata_link_get_uri (_link);
+	} else {
+		/* Default URI */
+		return "https://picasaweb.google.com/data/feed/api/user/default/albumid/default";
+	}
+}
+
 /**
  * gdata_picasaweb_service_query_files:
  * @self: a #GDataPicasaWebService
  * @album: (allow-none): a #GDataPicasaWebAlbum from which to retrieve the files, or %NULL
  * @query: (allow-none): a #GDataQuery with the query parameters, or %NULL
- * @cancellable: optional #GCancellable object, or %NULL
- * @progress_callback: (scope call): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
+ * @progress_callback: (allow-none) (scope call) (closure progress_user_data): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
  * @progress_user_data: (closure): data to pass to the @progress_callback function
  * @error: a #GError, or %NULL
  *
@@ -333,7 +410,6 @@ GDataFeed *
 gdata_picasaweb_service_query_files (GDataPicasaWebService *self, GDataPicasaWebAlbum *album, GDataQuery *query, GCancellable *cancellable,
                                      GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error)
 {
-	/* TODO: Async variant */
 	const gchar *uri;
 
 	g_return_val_if_fail (GDATA_IS_PICASAWEB_SERVICE (self), NULL);
@@ -342,68 +418,58 @@ gdata_picasaweb_service_query_files (GDataPicasaWebService *self, GDataPicasaWeb
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-	if (album != NULL) {
-		GDataLink *_link = gdata_entry_look_up_link (GDATA_ENTRY (album), "http://schemas.google.com/g/2005#feed");
-		if (_link == NULL) {
-			/* Error */
-			g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
-			                     _("The album did not have a feed link."));
-			return NULL;
-		}
-		uri = gdata_link_get_uri (_link);
-	} else {
-		/* Default URI */
-		uri = "http://picasaweb.google.com/data/feed/api/user/default/albumid/default";
-	}
-
-	/* Execute the query */
-	return gdata_service_query (GDATA_SERVICE (self), uri, GDATA_QUERY (query), GDATA_TYPE_PICASAWEB_FILE, cancellable,
-	                            progress_callback, progress_user_data, error);
-}
-
-static GOutputStream *
-get_file_output_stream (GDataPicasaWebService *self, GDataPicasaWebAlbum *album, GDataPicasaWebFile *file_entry, GFile *file_data, GError **error)
-{
-	GFileInfo *file_info = NULL;
-	const gchar *slug = NULL, *content_type = NULL, *user_id = NULL, *album_id = NULL;
-	GOutputStream *output_stream;
-	gchar *upload_uri;
-
-	/* PicasaWeb allows you to post to a default Dropbox */
-	album_id = (album != NULL) ? gdata_entry_get_id (GDATA_ENTRY (album)) : "default";
-	user_id = gdata_service_get_username (GDATA_SERVICE (self));
-
-	file_info = g_file_query_info (file_data, "standard::display-name,standard::content-type", G_FILE_QUERY_INFO_NONE, NULL, error);
-	if (file_info == NULL)
+	uri = get_query_files_uri (album, error);
+	if (uri == NULL)
 		return NULL;
 
-	slug = g_file_info_get_display_name (file_info);
-	content_type = g_file_info_get_content_type (file_info);
-
-	/* Build the upload URI and upload stream */
-	upload_uri = _gdata_service_build_uri (TRUE, "http://picasaweb.google.com/data/feed/api/user/%s/albumid/%s", user_id, album_id);
-	output_stream = gdata_upload_stream_new (GDATA_SERVICE (self), SOUP_METHOD_POST, upload_uri, GDATA_ENTRY (file_entry), slug, content_type);
-	g_free (upload_uri);
-	g_object_unref (file_info);
-
-	return output_stream;
+	/* Execute the query */
+	return gdata_service_query (GDATA_SERVICE (self), get_picasaweb_authorization_domain (), uri, GDATA_QUERY (query), GDATA_TYPE_PICASAWEB_FILE,
+	                            cancellable, progress_callback, progress_user_data, error);
 }
 
-static GDataPicasaWebFile *
-parse_spliced_stream (GOutputStream *output_stream, GError **error)
+/**
+ * gdata_picasaweb_service_query_files_async: (skip)
+ * @self: a #GDataPicasaWebService
+ * @album: (allow-none): a #GDataPicasaWebAlbum from which to retrieve the files, or %NULL
+ * @query: (allow-none): a #GDataQuery with the query parameters, or %NULL
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
+ * @progress_callback: (allow-none) (closure progress_user_data): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
+ * @progress_user_data: (closure): data to pass to the @progress_callback function
+ * @callback: a #GAsyncReadyCallback to call when the query is finished
+ * @user_data: (closure): data to pass to the @callback function
+ *
+ * Queries the specified @album for a list of the files which match the given @query. If @album is %NULL and a user is authenticated with the service,
+ * the user's default album will be queried. @self, @album and @query are all reffed when this function is called, so can safely be unreffed after
+ * this function returns.
+ *
+ * For more details, see gdata_picasaweb_service_query_files(), which is the synchronous version of this function, and gdata_service_query_async(),
+ * which is the base asynchronous query function.
+ *
+ * Since: 0.8.0
+ **/
+void
+gdata_picasaweb_service_query_files_async (GDataPicasaWebService *self, GDataPicasaWebAlbum *album, GDataQuery *query, GCancellable *cancellable,
+                                           GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GAsyncReadyCallback callback,
+                                           gpointer user_data)
 {
-	const gchar *response_body;
-	gssize response_length;
-	GDataPicasaWebFile *new_entry;
+	const gchar *request_uri;
+	GError *child_error = NULL;
 
-	/* Get the response from the server */
-	response_body = gdata_upload_stream_get_response (GDATA_UPLOAD_STREAM (output_stream), &response_length);
-	g_assert (response_body != NULL && response_length > 0);
+	g_return_if_fail (GDATA_IS_PICASAWEB_SERVICE (self));
+	g_return_if_fail (album == NULL || GDATA_IS_PICASAWEB_ALBUM (album));
+	g_return_if_fail (query == NULL || GDATA_IS_QUERY (query));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+	g_return_if_fail (callback != NULL);
 
-	/* Parse the response to produce a GDataPicasaWebFile */
-	new_entry = GDATA_PICASAWEB_FILE (gdata_parsable_new_from_xml (GDATA_TYPE_PICASAWEB_FILE, response_body, (gint) response_length, error));
+	request_uri = get_query_files_uri (album, &child_error);
+	if (request_uri == NULL) {
+		g_simple_async_report_gerror_in_idle (G_OBJECT (self), callback, user_data, child_error);
+		g_error_free (child_error);
+		return;
+	}
 
-	return new_entry;
+	gdata_service_query_async (GDATA_SERVICE (self), get_picasaweb_authorization_domain (), request_uri, GDATA_QUERY (query),
+	                           GDATA_TYPE_PICASAWEB_FILE, cancellable, progress_callback, progress_user_data, callback, user_data);
 }
 
 /**
@@ -411,36 +477,45 @@ parse_spliced_stream (GOutputStream *output_stream, GError **error)
  * @self: a #GDataPicasaWebService
  * @album: (allow-none): a #GDataPicasaWebAlbum into which to insert the file, or %NULL
  * @file_entry: a #GDataPicasaWebFile to insert
- * @file_data: the actual file to upload
- * @cancellable: optional #GCancellable object, or %NULL
+ * @slug: the filename to give to the uploaded file
+ * @content_type: the content type of the uploaded data
+ * @cancellable: (allow-none): a #GCancellable for the entire upload stream, or %NULL
  * @error: a #GError, or %NULL
  *
- * Uploads a file (photo or video) to the given PicasaWeb @album, using the @actual_file from disk and the metadata from @file. If @album is
- * %NULL, the file will be uploaded to the currently-authenticated user's "Drop Box" album. A user must be authenticated to use this function.
+ * Uploads a file (photo or video) to the given PicasaWeb @album, using the metadata from @file and the file data written to the resulting
+ * #GDataUploadStream. If @album is %NULL, the file will be uploaded to the currently-authenticated user's "Drop Box" album. A user must be
+ * authenticated to use this function.
  *
  * If @file has already been inserted, a %GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED error will be returned. If no user is authenticated
  * with the service, %GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED will be returned.
  *
- * If there is a problem reading @file_data, an error from g_output_stream_splice() or g_file_query_info() will be returned. Other errors from
- * #GDataServiceError can be returned for other exceptional conditions, as determined by the server.
+ * The stream returned by this function should be written to using the standard #GOutputStream methods, asychronously or synchronously. Once the stream
+ * is closed (using g_output_stream_close()), gdata_picasaweb_service_finish_file_upload() should be called on it to parse and return the updated
+ * #GDataPicasaWebFile for the uploaded file. This must be done, as @file_entry isn't updated in-place.
  *
- * Return value: (transfer full): the inserted #GDataPicasaWebFile with updated properties from @file_entry; unref with g_object_unref()
+ * In order to cancel the upload, a #GCancellable passed in to @cancellable must be cancelled using g_cancellable_cancel(). Cancelling the individual
+ * #GOutputStream operations on the #GDataUploadStream will not cancel the entire upload; merely the write or close operation in question. See the
+ * #GDataUploadStream:cancellable for more details.
  *
- * Since: 0.4.0
+ * Any upload errors will be thrown by the stream methods, and may come from the #GDataServiceError domain.
+ *
+ * Return value: (transfer full): a #GDataUploadStream to write the file data to, or %NULL; unref with g_object_unref()
+ *
+ * Since: 0.8.0
  **/
-GDataPicasaWebFile *
-gdata_picasaweb_service_upload_file (GDataPicasaWebService *self, GDataPicasaWebAlbum *album, GDataPicasaWebFile *file_entry, GFile *file_data,
-                                     GCancellable *cancellable, GError **error)
+GDataUploadStream *
+gdata_picasaweb_service_upload_file (GDataPicasaWebService *self, GDataPicasaWebAlbum *album, GDataPicasaWebFile *file_entry, const gchar *slug,
+                                     const gchar *content_type, GCancellable *cancellable, GError **error)
 {
-	GOutputStream *output_stream;
-	GInputStream *input_stream;
-	GDataPicasaWebFile *new_entry;
-	GError *child_error = NULL;
+	const gchar *album_id = NULL;
+	GDataUploadStream *upload_stream;
+	gchar *upload_uri;
 
 	g_return_val_if_fail (GDATA_IS_PICASAWEB_SERVICE (self), NULL);
 	g_return_val_if_fail (album == NULL || GDATA_IS_PICASAWEB_ALBUM (album), NULL);
 	g_return_val_if_fail (GDATA_IS_PICASAWEB_FILE (file_entry), NULL);
-	g_return_val_if_fail (G_IS_FILE (file_data), NULL);
+	g_return_val_if_fail (slug != NULL && *slug != '\0', NULL);
+	g_return_val_if_fail (content_type != NULL && *content_type != '\0', NULL);
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
@@ -450,207 +525,65 @@ gdata_picasaweb_service_upload_file (GDataPicasaWebService *self, GDataPicasaWeb
 		return NULL;
 	}
 
-	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
+	if (gdata_authorizer_is_authorized_for_domain (gdata_service_get_authorizer (GDATA_SERVICE (self)),
+	                                               get_picasaweb_authorization_domain ()) == FALSE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
 		                     _("You must be authenticated to upload a file."));
 		return NULL;
 	}
 
-	output_stream = get_file_output_stream (self, album, file_entry, file_data, error);
-	if (output_stream == NULL)
-		return NULL;
+	/* PicasaWeb allows you to post to a default Dropbox */
+	album_id = (album != NULL) ? gdata_entry_get_id (GDATA_ENTRY (album)) : "default";
 
-	/* Pipe the input file to the upload stream */
-	input_stream = G_INPUT_STREAM (g_file_read (file_data, cancellable, error));
-	if (input_stream == NULL) {
-		g_object_unref (output_stream);
-		return NULL;
-	}
+	/* Build the upload URI and upload stream */
+	upload_uri = _gdata_service_build_uri ("https://picasaweb.google.com/data/feed/api/user/default/albumid/%s", album_id);
+	upload_stream = GDATA_UPLOAD_STREAM (gdata_upload_stream_new (GDATA_SERVICE (self), get_picasaweb_authorization_domain (), SOUP_METHOD_POST,
+	                                                              upload_uri, GDATA_ENTRY (file_entry), slug, content_type, cancellable));
+	g_free (upload_uri);
 
-	g_output_stream_splice (output_stream, input_stream, G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
-	                        cancellable, &child_error);
-
-	g_object_unref (input_stream);
-	if (child_error != NULL) {
-		g_object_unref (output_stream);
-		g_propagate_error (error, child_error);
-		return NULL;
-	}
-
-	new_entry = parse_spliced_stream (output_stream, error);
-	g_object_unref (output_stream);
-
-	return new_entry;
+	return upload_stream;
 }
 
 /**
- * gdata_picasaweb_service_upload_file_finish:
+ * gdata_picasaweb_service_finish_file_upload:
  * @self: a #GDataPicasaWebService
- * @result: a #GSimpleAsyncResult
+ * @upload_stream: the #GDataUploadStream from the operation
  * @error: a #GError, or %NULL
  *
- * This should be called to obtain the result of a call to
- * gdata_picasaweb_service_upload_file_async() and to check for
- * errors.
+ * Finish off a file upload operation started by gdata_picasaweb_service_upload_file(), parsing the result and returning the new #GDataPicasaWebFile.
  *
- * If there is a problem reading the subect file's data, an error
- * from g_output_stream_splice() or g_file_query_info() will be
- * returned. Other errors from #GDataServiceError can be returned for
- * other exceptional conditions, as determined by the server.
+ * If an error occurred during the upload operation, it will have been returned during the operation (e.g. by g_output_stream_splice() or one
+ * of the other stream methods). In such a case, %NULL will be returned but @error will remain unset. @error is only set in the case that the server
+ * indicates that the operation was successful, but an error is encountered in parsing the result sent by the server.
  *
- * If the file to upload has already been inserted, a
- * %GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED error will be set. If
- * no user is authenticated with the service when trying to upload it,
- * %GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED will be set.
+ * Return value: (transfer full): the new #GDataPicasaWebFile, or %NULL; unref with g_object_unref()
  *
- * Return value: (transfer full): the inserted #GDataPicasaWebFile; unref with
- * g_object_unref()
- *
- * Since: 0.6.0
+ * Since: 0.8.0
  */
 GDataPicasaWebFile *
-gdata_picasaweb_service_upload_file_finish (GDataPicasaWebService *self, GAsyncResult *result, GError **error)
+gdata_picasaweb_service_finish_file_upload (GDataPicasaWebService *self, GDataUploadStream *upload_stream, GError **error)
 {
+	const gchar *response_body;
+	gssize response_length;
+
 	g_return_val_if_fail (GDATA_IS_PICASAWEB_SERVICE (self), NULL);
-	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), NULL);
+	g_return_val_if_fail (GDATA_IS_UPLOAD_STREAM (upload_stream), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
+	/* Get the response from the server */
+	response_body = gdata_upload_stream_get_response (upload_stream, &response_length);
+	if (response_body == NULL || response_length == 0)
 		return NULL;
 
-	g_assert (gdata_picasaweb_service_upload_file_async == g_simple_async_result_get_source_tag (G_SIMPLE_ASYNC_RESULT (result)));
-
-	return GDATA_PICASAWEB_FILE (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (result)));
-}
-
-typedef struct {
-	GDataPicasaWebService *service;
-	GAsyncReadyCallback callback;
-	gpointer user_data;
-} UploadFileAsyncData;
-
-static void
-upload_file_async_data_free (UploadFileAsyncData *data)
-{
-	g_object_unref (data->service);
-	g_slice_free (UploadFileAsyncData, data);
-}
-
-static void
-upload_file_async_cb (GOutputStream *output_stream, GAsyncResult *result, UploadFileAsyncData *data)
-{
-	GError *error = NULL;
-	GDataPicasaWebFile *file = NULL;
-	GSimpleAsyncResult *async_result;
-
-	g_output_stream_splice_finish (output_stream, result, &error);
-
-	/* If we're error free, parse the file from the stream */
-	if (error == NULL)
-		file = parse_spliced_stream (output_stream, &error);
-
-	if (error == NULL && file != NULL) {
-		async_result = g_simple_async_result_new (G_OBJECT (data->service), (GAsyncReadyCallback) data->callback,
-		                                          data->user_data, gdata_picasaweb_service_upload_file_async);
-	} else {
-		async_result = g_simple_async_result_new_from_error (G_OBJECT (data->service), (GAsyncReadyCallback) data->callback,
-		                                                     data->user_data, error);
-	}
-
-	g_simple_async_result_set_op_res_gpointer (async_result, file, NULL);
-
-	g_simple_async_result_complete (async_result);
-
-	upload_file_async_data_free (data);
-	g_object_unref (async_result);
-}
-
-/**
- * gdata_picasaweb_service_upload_file_async:
- * @self: a #GDataPicasaWebService
- * @album: (allow-none): a #GDataPicasaWebAlbum into which to insert the file, or %NULL
- * @file_entry: a #GDataPicasaWebFile to insert
- * @file_data: the actual file to upload
- * @cancellable: optional #GCancellable object, or %NULL
- * @callback: a #GAsyncReadyCallback to call when authentication is finished
- * @user_data: (closure): data to pass to the @callback function
- *
- * Uploads a file (photo or video) to the given PicasaWeb @album
- * asynchronously, using the @actual_file from disk and the metadata
- * from @file. If @album is %NULL, the file will be uploaded to the
- * currently-authenticated user's "Drop Box" album. A user must be
- * authenticated to use this function.
- *
- * @callback should call gdata_picasaweb_service_upload_file_finish()
- * to obtain a #GDataPicasaWebFile representing the uploaded file and
- * check for possible errors.
- *
- * Since: 0.6.0
- **/
-void
-gdata_picasaweb_service_upload_file_async (GDataPicasaWebService *self, GDataPicasaWebAlbum *album, GDataPicasaWebFile *file_entry,
-                                           GFile *file_data, GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data)
-{
-	GOutputStream *output_stream;
-	GInputStream *input_stream;
-	UploadFileAsyncData *data;
-	GSimpleAsyncResult *result;
-	GError *error = NULL;
-
-	g_return_if_fail (GDATA_IS_PICASAWEB_SERVICE (self));
-	g_return_if_fail (album == NULL || GDATA_IS_PICASAWEB_ALBUM (album));
-	g_return_if_fail (GDATA_IS_PICASAWEB_FILE (file_entry));
-	g_return_if_fail (G_IS_FILE (file_data));
-	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
-
-	if (gdata_entry_is_inserted (GDATA_ENTRY (file_entry)) == TRUE) {
-		g_set_error_literal (&error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED,
-		                     _("The entry has already been inserted."));
-		goto error;
-	}
-
-	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
-		g_set_error_literal (&error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-		                     _("You must be authenticated to upload a file."));
-		goto error;
-	}
-
-	/* Prepare and retrieve a #GDataOutputStream for the file and its data */
-	output_stream = get_file_output_stream (self, album, file_entry, file_data, &error);
-	if (output_stream == NULL)
-		goto error;
-
-	/* Pipe the input file to the upload stream */
-	input_stream = G_INPUT_STREAM (g_file_read (file_data, cancellable, &error));
-	if (input_stream == NULL) {
-		g_object_unref (output_stream);
-		goto error;
-	}
-
-	data = g_slice_new (UploadFileAsyncData);
-	data->service = g_object_ref (self);
-	data->callback = callback;
-	data->user_data = user_data;
-
-	/* Actually transfer the data */
-	g_output_stream_splice_async (output_stream, input_stream, G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
-	                              0, cancellable, (GAsyncReadyCallback) upload_file_async_cb, data);
-
-	g_object_unref (input_stream);
-	g_object_unref (output_stream);
-
-	return;
-
-error:
-	result = g_simple_async_result_new_from_error (G_OBJECT (self), callback, user_data, error);
-	g_simple_async_result_complete (result);
+	/* Parse the response to produce a GDataPicasaWebFile */
+	return GDATA_PICASAWEB_FILE (gdata_parsable_new_from_xml (GDATA_TYPE_PICASAWEB_FILE, response_body, (gint) response_length, error));
 }
 
 /**
  * gdata_picasaweb_service_insert_album:
  * @self: a #GDataPicasaWebService
  * @album: a #GDataPicasaWebAlbum to create on the server
- * @cancellable: optional #GCancellable object, or %NULL
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
  * @error: a #GError, or %NULL
  *
  * Inserts a new album described by @album. A user must be
@@ -675,13 +608,46 @@ gdata_picasaweb_service_insert_album (GDataPicasaWebService *self, GDataPicasaWe
 		return NULL;
 	}
 
-	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
+	if (gdata_authorizer_is_authorized_for_domain (gdata_service_get_authorizer (GDATA_SERVICE (self)),
+	                                               get_picasaweb_authorization_domain ()) == FALSE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
 		                     _("You must be authenticated to insert an album."));
 		return NULL;
 	}
 
-	return GDATA_PICASAWEB_ALBUM (gdata_service_insert_entry (GDATA_SERVICE (self), "http://picasaweb.google.com/data/feed/api/user/default",
+	return GDATA_PICASAWEB_ALBUM (gdata_service_insert_entry (GDATA_SERVICE (self), get_picasaweb_authorization_domain (),
+	                                                          "https://picasaweb.google.com/data/feed/api/user/default",
 	                                                          GDATA_ENTRY (album), cancellable, error));
 }
 
+/**
+ * gdata_picasaweb_service_insert_album_async:
+ * @self: a #GDataPicasaWebService
+ * @album: a #GDataPicasaWebAlbum to create on the server
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
+ * @callback: a #GAsyncReadyCallback to call when insertion is finished
+ * @user_data: (closure): data to pass to the @callback function
+ *
+ * Inserts a new album described by @album. The user must be authenticated to use this function. @self and @album are both reffed when this function
+ * is called, so can safely be unreffed after this function returns.
+ *
+ * @callback should call gdata_service_insert_entry_finish() to obtain a #GDataPicasaWebAlbum representing the inserted album and to check for
+ * possible errors.
+ *
+ * For more details, see gdata_picasaweb_service_insert_album(), which is the synchronous version of this function, and
+ * gdata_service_insert_entry_async(), which is the base asynchronous insertion function.
+ *
+ * Since: 0.8.0
+ **/
+void
+gdata_picasaweb_service_insert_album_async (GDataPicasaWebService *self, GDataPicasaWebAlbum *album, GCancellable *cancellable,
+                                            GAsyncReadyCallback callback, gpointer user_data)
+{
+	g_return_if_fail (GDATA_IS_PICASAWEB_SERVICE (self));
+	g_return_if_fail (GDATA_IS_PICASAWEB_ALBUM (album));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+
+	gdata_service_insert_entry_async (GDATA_SERVICE (self), get_picasaweb_authorization_domain (),
+	                                  "https://picasaweb.google.com/data/feed/api/user/default", GDATA_ENTRY (album), cancellable, callback,
+	                                  user_data);
+}
