@@ -58,6 +58,144 @@
  *	g_object_unref (service);
  * 	</programlisting>
  * </example>
+ *
+ * <example>
+ * 	<title>Uploading a Video from Disk</title>
+ * 	<programlisting>
+ *	GDataYouTubeService *service;
+ *	GDataYouTubeVideo *video, *uploaded_video;
+ *	GDataMediaCategory *category;
+ *	const gchar * const tags[] = { "tag1", "tag2", NULL };
+ *	GFile *video_file;
+ *	GFileInfo *file_info;
+ *	const gchar *slug, *content_type;
+ *	GFileInputStream *file_stream;
+ *	GDataUploadStream *upload_stream;
+ *	GError *error = NULL;
+ *
+ *	/<!-- -->* Create a service *<!-- -->/
+ *	service = create_youtube_service ();
+ *
+ *	/<!-- -->* Get the video file to upload *<!-- -->/
+ *	video_file = g_file_new_for_path ("sample.ogg");
+ *
+ *	/<!-- -->* Get the file's display name and content type *<!-- -->/
+ *	file_info = g_file_query_info (video_file, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME "," G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+ *	                               G_FILE_QUERY_INFO_NONE, NULL, &error);
+ *
+ *	if (error != NULL) {
+ *		g_error ("Error getting video file information: %s", error->message);
+ *		g_error_free (error);
+ *		g_object_unref (video_file);
+ *		g_object_unref (service);
+ *		return;
+ *	}
+ *
+ *	slug = g_file_info_get_display_name (file_info);
+ *	content_type = g_file_info_get_content_type (file_info);
+ *
+ *	/<!-- -->* Get an input stream for the file *<!-- -->/
+ *	file_stream = g_file_read (video_file, NULL, &error);
+ *
+ *	g_object_unref (video_file);
+ *
+ *	if (error != NULL) {
+ *		g_error ("Error getting video file stream: %s", error->message);
+ *		g_error_free (error);
+ *		g_object_unref (file_info);
+ *		g_object_unref (service);
+ *		return;
+ *	}
+ *
+ *	/<!-- -->* Create the video to upload *<!-- -->/
+ *	video = gdata_youtube_video_new (NULL);
+ *
+ *	gdata_entry_set_title (GDATA_ENTRY (video), "Video Title");
+ *	gdata_youtube_video_set_description (video, "Video description.");
+ *	gdata_youtube_video_set_keywords (video, video_tags);
+ *
+ *	category = gdata_media_category_new ("People", "http://gdata.youtube.com/schemas/2007/categories.cat", NULL);
+ *	gdata_youtube_video_set_category (video, category);
+ *	g_object_unref (category);
+ *
+ *	/<!-- -->* Get an upload stream for the video *<!-- -->/
+ *	upload_stream = gdata_youtube_service_upload_video (service, video, slug, content_type, NULL, &error);
+ *
+ *	g_object_unref (video);
+ *	g_object_unref (file_info);
+ *
+ *	if (error != NULL) {
+ *		g_error ("Error getting upload stream: %s", error->message);
+ *		g_error_free (error);
+ *		g_object_unref (file_stream);
+ *		g_object_unref (service);
+ *		return;
+ *	}
+ *
+ *	/<!-- -->* Upload the video. This is a blocking operation, and should normally be done asynchronously. *<!-- -->/
+ *	g_output_stream_splice (G_OUTPUT_STREAM (upload_stream), G_INPUT_STREAM (file_stream),
+ *	                        G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET, NULL, &error);
+ *
+ *	g_object_unref (file_stream);
+ *
+ *	if (error != NULL) {
+ *		g_error ("Error splicing streams: %s", error->message);
+ *		g_error_free (error);
+ *		g_object_unref (upload_stream);
+ *		g_object_unref (service);
+ *		return;
+ *	}
+ *
+ *	/<!-- -->* Finish off the upload by parsing the returned updated video entry *<!-- -->/
+ *	uploaded_video = gdata_youtube_service_finish_video_upload (service, upload_stream, &error);
+ *
+ *	g_object_unref (upload_stream);
+ *	g_object_unref (service);
+ *
+ *	if (error != NULL) {
+ *		g_error ("Error uploading video: %s", error->message);
+ *		g_error_free (error);
+ *		return;
+ *	}
+ *
+ *	/<!-- -->* Do something with the uploaded video *<!-- -->/
+ *
+ *	g_object_unref (uploaded_video);
+ * 	</programlisting>
+ * </example>
+ *
+ * <example>
+ * 	<title>Querying for Videos from a Standard Feed</title>
+ * 	<programlisting>
+ *	GDataYouTubeService *service;
+ *	GDataFeed *feed;
+ *	GList *i;
+ *	GError *error = NULL;
+ *
+ *	/<!-- -->* Create a service *<!-- -->/
+ *	service = create_youtube_service ();
+ *
+ *	/<!-- -->* Query for the top page of videos in the most popular feed *<!-- -->/
+ *	feed = gdata_youtube_service_query_standard_feed (service, GDATA_YOUTUBE_MOST_POPULAR_FEED, NULL, NULL, NULL, NULL, &error);
+ *
+ *	g_object_unref (service);
+ *
+ *	if (error != NULL) {
+ *		g_error ("Error querying for most popular videos: %s", error->message);
+ *		g_error_free (error);
+ *		return;
+ *	}
+ *
+ *	/<!-- -->* Iterate through the videos *<!-- -->/
+ *	for (i = gdata_feed_get_entries (feed); i != NULL; i = i->next) {
+ *		GDataYouTubeVideo *video = GDATA_YOUTUBE_VIDEO (i->data);
+ *
+ *		/<!-- -->* Do something with the video, like insert it into the UI *<!-- -->/
+ *	}
+ *
+ *	g_object_unref (feed);
+ * 	</programlisting>
+ * </example>
  **/
 
 #include <config.h>
@@ -86,21 +224,21 @@ gdata_youtube_service_error_quark (void)
 static void gdata_youtube_service_finalize (GObject *object);
 static void gdata_youtube_service_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_youtube_service_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
-static gboolean parse_authentication_response (GDataService *self, guint status, const gchar *response_body, gint length, GError **error);
-static void append_query_headers (GDataService *self, SoupMessage *message);
+static void append_query_headers (GDataService *self, GDataAuthorizationDomain *domain, SoupMessage *message);
 static void parse_error_response (GDataService *self, GDataOperationType operation_type, guint status, const gchar *reason_phrase,
                                   const gchar *response_body, gint length, GError **error);
 
+static GList *get_authorization_domains (void);
+
 struct _GDataYouTubeServicePrivate {
-	gchar *youtube_user;
 	gchar *developer_key;
 };
 
 enum {
-	PROP_DEVELOPER_KEY = 1,
-	PROP_YOUTUBE_USER
+	PROP_DEVELOPER_KEY = 1
 };
 
+_GDATA_DEFINE_AUTHORIZATION_DOMAIN (youtube, "youtube", "http://gdata.youtube.com")
 G_DEFINE_TYPE_WITH_CODE (GDataYouTubeService, gdata_youtube_service, GDATA_TYPE_SERVICE, G_IMPLEMENT_INTERFACE (GDATA_TYPE_BATCHABLE, NULL))
 
 static void
@@ -115,37 +253,21 @@ gdata_youtube_service_class_init (GDataYouTubeServiceClass *klass)
 	gobject_class->get_property = gdata_youtube_service_get_property;
 	gobject_class->finalize = gdata_youtube_service_finalize;
 
-	service_class->service_name = "youtube";
-	service_class->authentication_uri = "https://www.google.com/youtube/accounts/ClientLogin";
-	service_class->parse_authentication_response = parse_authentication_response;
 	service_class->append_query_headers = append_query_headers;
 	service_class->parse_error_response = parse_error_response;
+	service_class->get_authorization_domains = get_authorization_domains;
 
 	/**
 	 * GDataYouTubeService:developer-key:
 	 *
 	 * The developer key your application has registered with the YouTube API. For more information, see the <ulink type="http"
 	 * url="http://code.google.com/apis/youtube/2.0/developers_guide_protocol.html#Developer_Key">online documentation</ulink>.
-	 *
-	 * The matching #GDataService:client-id property belongs to #GDataService.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_DEVELOPER_KEY,
 	                                 g_param_spec_string ("developer-key",
 	                                                      "Developer key", "Your YouTube developer API key.",
 	                                                      NULL,
 	                                                      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-	/**
-	 * GDataYouTubeService:youtube-user:
-	 *
-	 * The YouTube username of the authenticated user, or %NULL. This may differ from #GDataService:username, due to the work done when
-	 * YouTube was converted to use Google's centralised login system.
-	 **/
-	g_object_class_install_property (gobject_class, PROP_YOUTUBE_USER,
-	                                 g_param_spec_string ("youtube-user",
-	                                                      "YouTube username", "The YouTube account username.",
-	                                                      NULL,
-	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -159,7 +281,6 @@ gdata_youtube_service_finalize (GObject *object)
 {
 	GDataYouTubeServicePrivate *priv = GDATA_YOUTUBE_SERVICE (object)->priv;
 
-	g_free (priv->youtube_user);
 	g_free (priv->developer_key);
 
 	/* Chain up to the parent class */
@@ -175,9 +296,6 @@ gdata_youtube_service_get_property (GObject *object, guint property_id, GValue *
 		case PROP_DEVELOPER_KEY:
 			g_value_set_string (value, priv->developer_key);
 			break;
-		case PROP_YOUTUBE_USER:
-			g_value_set_string (value, priv->youtube_user);
-			break;
 		default:
 			/* We don't have any other property... */
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -201,42 +319,8 @@ gdata_youtube_service_set_property (GObject *object, guint property_id, const GV
 	}
 }
 
-static gboolean
-parse_authentication_response (GDataService *self, guint status, const gchar *response_body, gint length, GError **error)
-{
-	GDataYouTubeServicePrivate *priv = GDATA_YOUTUBE_SERVICE (self)->priv;
-	gchar *user_start, *user_end;
-
-	/* Chain up to the parent method first */
-	if (GDATA_SERVICE_CLASS (gdata_youtube_service_parent_class)->parse_authentication_response (self, status,
-		                                                                                     response_body, length, error) == FALSE) {
-		return FALSE;
-	}
-
-	/* Parse the response */
-	user_start = strstr (response_body, "YouTubeUser=");
-	if (user_start == NULL)
-		goto protocol_error;
-	user_start += strlen ("YouTubeUser=");
-
-	user_end = strstr (user_start, "\n");
-	if (user_end == NULL)
-		goto protocol_error;
-
-	priv->youtube_user = g_strndup (user_start, user_end - user_start);
-	if (priv->youtube_user == NULL || strlen (priv->youtube_user) == 0)
-		goto protocol_error;
-
-	return TRUE;
-
-protocol_error:
-	g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
-	                     _("The server returned a malformed response."));
-	return FALSE;
-}
-
 static void
-append_query_headers (GDataService *self, SoupMessage *message)
+append_query_headers (GDataService *self, GDataAuthorizationDomain *domain, SoupMessage *message)
 {
 	GDataYouTubeServicePrivate *priv = GDATA_YOUTUBE_SERVICE (self)->priv;
 	gchar *key_header;
@@ -249,7 +333,7 @@ append_query_headers (GDataService *self, SoupMessage *message)
 	g_free (key_header);
 
 	/* Chain up to the parent class */
-	GDATA_SERVICE_CLASS (gdata_youtube_service_parent_class)->append_query_headers (self, message);
+	GDATA_SERVICE_CLASS (gdata_youtube_service_parent_class)->append_query_headers (self, domain, message);
 }
 
 static void
@@ -332,8 +416,7 @@ parse_error_response (GDataService *self, GDataOperationType operation_type, gui
 				g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_UNAVAILABLE,
 				             _("This service is not available at the moment."));
 			} else if (xmlStrcmp (domain, (xmlChar*) "yt:authentication") == 0) {
-				/* Authentication problem; make sure to set our status as unauthenticated */
-				_gdata_service_set_authenticated (GDATA_SERVICE (self), FALSE);
+				/* Authentication problem */
 				g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
 				             _("You must be authenticated to do this."));
 			} else if (xmlStrcmp (domain, (xmlChar*) "yt:quota") == 0) {
@@ -384,52 +467,80 @@ parent:
 	                                                                                response_body, length, error);
 }
 
+static GList *
+get_authorization_domains (void)
+{
+	return g_list_prepend (NULL, get_youtube_authorization_domain ());
+}
+
 /**
  * gdata_youtube_service_new:
  * @developer_key: your application's developer API key
- * @client_id: your application's client ID
+ * @authorizer: (allow-none): a #GDataAuthorizer to authorize the service's requests, or %NULL
  *
- * Creates a new #GDataYouTubeService. The @developer_key and @client_id must be unique for your application, and as
+ * Creates a new #GDataYouTubeService using the given #GDataAuthorizer. If @authorizer is %NULL, all requests are made as an unauthenticated user.
+ * The @developer_key must be unique for your application, and as
  * <ulink type="http" url="http://code.google.com/apis/youtube/2.0/developers_guide_protocol.html#Developer_Key">registered with Google</ulink>.
  *
- * Return value: a new #GDataYouTubeService, or %NULL
- **/
+ * Return value: a new #GDataYouTubeService, or %NULL; unref with g_object_unref()
+ *
+ * Since: 0.9.0
+ */
 GDataYouTubeService *
-gdata_youtube_service_new (const gchar *developer_key, const gchar *client_id)
+gdata_youtube_service_new (const gchar *developer_key, GDataAuthorizer *authorizer)
 {
 	g_return_val_if_fail (developer_key != NULL, NULL);
-	g_return_val_if_fail (client_id != NULL, NULL);
+	g_return_val_if_fail (authorizer == NULL || GDATA_IS_AUTHORIZER (authorizer), NULL);
 
 	return g_object_new (GDATA_TYPE_YOUTUBE_SERVICE,
 	                     "developer-key", developer_key,
-	                     "client-id", client_id,
+	                     "authorizer", authorizer,
 	                     NULL);
 }
 
+/**
+ * gdata_youtube_service_get_primary_authorization_domain:
+ *
+ * The primary #GDataAuthorizationDomain for interacting with YouTube. This will not normally need to be used, as it's used internally
+ * by the #GDataYouTubeService methods. However, if using the plain #GDataService methods to implement custom queries or requests which libgdata
+ * does not support natively, then this domain may be needed to authorize the requests.
+ *
+ * The domain never changes, and is interned so that pointer comparison can be used to differentiate it from other authorization domains.
+ *
+ * Return value: (transfer none): the service's authorization domain
+ *
+ * Since: 0.9.0
+ */
+GDataAuthorizationDomain *
+gdata_youtube_service_get_primary_authorization_domain (void)
+{
+	return get_youtube_authorization_domain ();
+}
+
 static const gchar *
 standard_feed_type_to_feed_uri (GDataYouTubeStandardFeedType feed_type)
 {
 	switch (feed_type) {
 	case GDATA_YOUTUBE_TOP_RATED_FEED:
-		return "http://gdata.youtube.com/feeds/api/standardfeeds/top_rated";
+		return "https://gdata.youtube.com/feeds/api/standardfeeds/top_rated";
 	case GDATA_YOUTUBE_TOP_FAVORITES_FEED:
-		return "http://gdata.youtube.com/feeds/api/standardfeeds/top_favorites";
+		return "https://gdata.youtube.com/feeds/api/standardfeeds/top_favorites";
 	case GDATA_YOUTUBE_MOST_VIEWED_FEED:
-		return "http://gdata.youtube.com/feeds/api/standardfeeds/most_viewed";
+		return "https://gdata.youtube.com/feeds/api/standardfeeds/most_viewed";
 	case GDATA_YOUTUBE_MOST_POPULAR_FEED:
-		return "http://gdata.youtube.com/feeds/api/standardfeeds/most_popular";
+		return "https://gdata.youtube.com/feeds/api/standardfeeds/most_popular";
 	case GDATA_YOUTUBE_MOST_RECENT_FEED:
-		return "http://gdata.youtube.com/feeds/api/standardfeeds/most_recent";
+		return "https://gdata.youtube.com/feeds/api/standardfeeds/most_recent";
 	case GDATA_YOUTUBE_MOST_DISCUSSED_FEED:
-		return "http://gdata.youtube.com/feeds/api/standardfeeds/most_discussed";
+		return "https://gdata.youtube.com/feeds/api/standardfeeds/most_discussed";
 	case GDATA_YOUTUBE_MOST_LINKED_FEED:
-		return "http://gdata.youtube.com/feeds/api/standardfeeds/most_linked";
+		return "https://gdata.youtube.com/feeds/api/standardfeeds/most_linked";
 	case GDATA_YOUTUBE_MOST_RESPONDED_FEED:
-		return "http://gdata.youtube.com/feeds/api/standardfeeds/most_responded";
+		return "https://gdata.youtube.com/feeds/api/standardfeeds/most_responded";
 	case GDATA_YOUTUBE_RECENTLY_FEATURED_FEED:
-		return "http://gdata.youtube.com/feeds/api/standardfeeds/recently_featured";
+		return "https://gdata.youtube.com/feeds/api/standardfeeds/recently_featured";
 	case GDATA_YOUTUBE_WATCH_ON_MOBILE_FEED:
-		return "http://gdata.youtube.com/feeds/api/standardfeeds/watch_on_mobile";
+		return "https://gdata.youtube.com/feeds/api/standardfeeds/watch_on_mobile";
 	default:
 		g_assert_not_reached ();
 	}
@@ -440,8 +551,8 @@ standard_feed_type_to_feed_uri (GDataYouTubeStandardFeedType feed_type)
  * @self: a #GDataYouTubeService
  * @feed_type: the feed type to query, from #GDataYouTubeStandardFeedType
  * @query: (allow-none): a #GDataQuery with the query parameters, or %NULL
- * @cancellable: optional #GCancellable object, or %NULL
- * @progress_callback: (scope call): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
+ * @progress_callback: (allow-none) (scope call) (closure progress_user_data): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
  * @progress_user_data: (closure): data to pass to the @progress_callback function
  * @error: a #GError, or %NULL
  *
@@ -462,7 +573,7 @@ gdata_youtube_service_query_standard_feed (GDataYouTubeService *self, GDataYouTu
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* TODO: Support the "time" parameter, as well as category- and region-specific feeds */
-	return gdata_service_query (GDATA_SERVICE (self), standard_feed_type_to_feed_uri (feed_type), query,
+	return gdata_service_query (GDATA_SERVICE (self), get_youtube_authorization_domain (), standard_feed_type_to_feed_uri (feed_type), query,
 	                            GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, error);
 }
 
@@ -471,8 +582,8 @@ gdata_youtube_service_query_standard_feed (GDataYouTubeService *self, GDataYouTu
  * @self: a #GDataService
  * @feed_type: the feed type to query, from #GDataYouTubeStandardFeedType
  * @query: (allow-none): a #GDataQuery with the query parameters, or %NULL
- * @cancellable: optional #GCancellable object, or %NULL
- * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
+ * @progress_callback: (allow-none) (closure progress_user_data): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
  * @progress_user_data: (closure): data to pass to the @progress_callback function
  * @callback: a #GAsyncReadyCallback to call when authentication is finished
  * @user_data: (closure): data to pass to the @callback function
@@ -495,7 +606,7 @@ gdata_youtube_service_query_standard_feed_async (GDataYouTubeService *self, GDat
 	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (callback != NULL);
 
-	gdata_service_query_async (GDATA_SERVICE (self), standard_feed_type_to_feed_uri (feed_type), query,
+	gdata_service_query_async (GDATA_SERVICE (self), get_youtube_authorization_domain (), standard_feed_type_to_feed_uri (feed_type), query,
 	                           GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, callback, user_data);
 }
 
@@ -503,8 +614,8 @@ gdata_youtube_service_query_standard_feed_async (GDataYouTubeService *self, GDat
  * gdata_youtube_service_query_videos:
  * @self: a #GDataYouTubeService
  * @query: (allow-none): a #GDataQuery with the query parameters, or %NULL
- * @cancellable: optional #GCancellable object, or %NULL
- * @progress_callback: (scope call): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
+ * @progress_callback: (allow-none) (scope call) (closure progress_user_data): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
  * @progress_user_data: (closure): data to pass to the @progress_callback function
  * @error: a #GError, or %NULL
  *
@@ -525,7 +636,7 @@ gdata_youtube_service_query_videos (GDataYouTubeService *self, GDataQuery *query
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-	return gdata_service_query (GDATA_SERVICE (self), "http://gdata.youtube.com/feeds/api/videos", query,
+	return gdata_service_query (GDATA_SERVICE (self), get_youtube_authorization_domain (), "https://gdata.youtube.com/feeds/api/videos", query,
 	                            GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, error);
 }
 
@@ -533,8 +644,8 @@ gdata_youtube_service_query_videos (GDataYouTubeService *self, GDataQuery *query
  * gdata_youtube_service_query_videos_async: (skip)
  * @self: a #GDataService
  * @query: (allow-none): a #GDataQuery with the query parameters, or %NULL
- * @cancellable: optional #GCancellable object, or %NULL
- * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
+ * @progress_callback: (allow-none) (closure progress_user_data): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
  * @progress_user_data: (closure): data to pass to the @progress_callback function
  * @callback: a #GAsyncReadyCallback to call when authentication is finished
  * @user_data: (closure): data to pass to the @callback function
@@ -557,7 +668,7 @@ gdata_youtube_service_query_videos_async (GDataYouTubeService *self, GDataQuery
 	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (callback != NULL);
 
-	gdata_service_query_async (GDATA_SERVICE (self), "http://gdata.youtube.com/feeds/api/videos", query,
+	gdata_service_query_async (GDATA_SERVICE (self), get_youtube_authorization_domain (), "https://gdata.youtube.com/feeds/api/videos", query,
 	                           GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, callback, user_data);
 }
 
@@ -566,8 +677,8 @@ gdata_youtube_service_query_videos_async (GDataYouTubeService *self, GDataQuery
  * @self: a #GDataYouTubeService
  * @video: a #GDataYouTubeVideo for which to find related videos
  * @query: (allow-none): a #GDataQuery with the query parameters, or %NULL
- * @cancellable: optional #GCancellable object, or %NULL
- * @progress_callback: (scope call): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
+ * @progress_callback: (allow-none) (scope call) (closure progress_user_data): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
  * @progress_user_data: (closure): data to pass to the @progress_callback function
  * @error: a #GError, or %NULL
  *
@@ -584,6 +695,8 @@ gdata_youtube_service_query_related (GDataYouTubeService *self, GDataYouTubeVide
                                      GError **error)
 {
 	GDataLink *related_link;
+	GDataFeed *feed;
+	gchar *uri;
 
 	g_return_val_if_fail (GDATA_IS_YOUTUBE_SERVICE (self), NULL);
 	g_return_val_if_fail (GDATA_IS_YOUTUBE_VIDEO (video), NULL);
@@ -601,8 +714,12 @@ gdata_youtube_service_query_related (GDataYouTubeService *self, GDataYouTubeVide
 	}
 
 	/* Execute the query */
-	return gdata_service_query (GDATA_SERVICE (self), gdata_link_get_uri (related_link), query,
+	uri = _gdata_service_fix_uri_scheme (gdata_link_get_uri (related_link));
+	feed = gdata_service_query (GDATA_SERVICE (self), get_youtube_authorization_domain (), uri, query,
 	                            GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, error);
+	g_free (uri);
+
+	return feed;
 }
 
 /**
@@ -610,8 +727,8 @@ gdata_youtube_service_query_related (GDataYouTubeService *self, GDataYouTubeVide
  * @self: a #GDataService
  * @video: a #GDataYouTubeVideo for which to find related videos
  * @query: (allow-none): a #GDataQuery with the query parameters, or %NULL
- * @cancellable: optional #GCancellable object, or %NULL
- * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
+ * @progress_callback: (allow-none) (closure progress_user_data): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
  * @progress_user_data: (closure): data to pass to the @progress_callback function
  * @callback: a #GAsyncReadyCallback to call when authentication is finished
  * @user_data: (closure): data to pass to the @callback function
@@ -630,6 +747,7 @@ gdata_youtube_service_query_related_async (GDataYouTubeService *self, GDataYouTu
                                            GAsyncReadyCallback callback, gpointer user_data)
 {
 	GDataLink *related_link;
+	gchar *uri;
 
 	g_return_if_fail (GDATA_IS_YOUTUBE_SERVICE (self));
 	g_return_if_fail (GDATA_IS_YOUTUBE_VIDEO (video));
@@ -647,43 +765,48 @@ gdata_youtube_service_query_related_async (GDataYouTubeService *self, GDataYouTu
 		return;
 	}
 
-	gdata_service_query_async (GDATA_SERVICE (self), gdata_link_get_uri (related_link), query,
+	uri = _gdata_service_fix_uri_scheme (gdata_link_get_uri (related_link));
+	gdata_service_query_async (GDATA_SERVICE (self), get_youtube_authorization_domain (), uri, query,
 	                           GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, callback, user_data);
+	g_free (uri);
 }
 
 /**
  * gdata_youtube_service_upload_video:
  * @self: a #GDataYouTubeService
  * @video: a #GDataYouTubeVideo to insert
- * @video_file: the video file to upload
- * @cancellable: optional #GCancellable object, or %NULL
+ * @slug: the filename to give to the uploaded file
+ * @content_type: the content type of the uploaded data
+ * @cancellable: (allow-none): a #GCancellable for the entire upload stream, or %NULL
  * @error: a #GError, or %NULL
  *
- * Uploads a video to YouTube, using the properties from @video and the video file pointed to by @video_file.
+ * Uploads a video to YouTube, using the properties from @video and the file data written to the resulting #GDataUploadStream.
  *
  * If @video has already been inserted, a %GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED error will be returned. If no user is authenticated
  * with the service, %GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED will be returned.
  *
- * If there is a problem reading @video_file, an error from g_file_load_contents() or g_file_query_info() will be returned. Other errors from
- * #GDataServiceError can be returned for other exceptional conditions, as determined by the server.
+ * The stream returned by this function should be written to using the standard #GOutputStream methods, asychronously or synchronously. Once the stream
+ * is closed (using g_output_stream_close()), gdata_youtube_service_finish_video_upload() should be called on it to parse and return the updated
+ * #GDataYouTubeVideo for the uploaded video. This must be done, as @video isn't updated in-place.
  *
- * Return value: (transfer full): the inserted #GDataYouTubeVideo with updated properties from @video; unref with g_object_unref()
+ * In order to cancel the upload, a #GCancellable passed in to @cancellable must be cancelled using g_cancellable_cancel(). Cancelling the individual
+ * #GOutputStream operations on the #GDataUploadStream will not cancel the entire upload; merely the write or close operation in question. See the
+ * #GDataUploadStream:cancellable for more details.
+ *
+ * Any upload errors will be thrown by the stream methods, and may come from the #GDataServiceError domain.
+ *
+ * Return value: (transfer full): a #GDataUploadStream to write the video data to, or %NULL; unref with g_object_unref()
+ *
+ * Since: 0.8.0
  **/
-GDataYouTubeVideo *
-gdata_youtube_service_upload_video (GDataYouTubeService *self, GDataYouTubeVideo *video, GFile *video_file, GCancellable *cancellable, GError **error)
+GDataUploadStream *
+gdata_youtube_service_upload_video (GDataYouTubeService *self, GDataYouTubeVideo *video, const gchar *slug, const gchar *content_type,
+                                    GCancellable *cancellable, GError **error)
 {
-	/* TODO: Async variant */
-	GDataYouTubeVideo *new_entry;
-	GOutputStream *output_stream;
-	GInputStream *input_stream;
-	const gchar *slug, *content_type, *response_body;
-	gssize response_length;
-	GFileInfo *file_info;
-	GError *child_error = NULL;
-
 	g_return_val_if_fail (GDATA_IS_YOUTUBE_SERVICE (self), NULL);
 	g_return_val_if_fail (GDATA_IS_YOUTUBE_VIDEO (video), NULL);
-	g_return_val_if_fail (G_IS_FILE (video_file), NULL);
+	g_return_val_if_fail (slug != NULL && *slug != '\0', NULL);
+	g_return_val_if_fail (content_type != NULL && *content_type != '\0', NULL);
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
@@ -693,54 +816,52 @@ gdata_youtube_service_upload_video (GDataYouTubeService *self, GDataYouTubeVideo
 		return NULL;
 	}
 
-	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
+	if (gdata_authorizer_is_authorized_for_domain (gdata_service_get_authorizer (GDATA_SERVICE (self)),
+	                                               get_youtube_authorization_domain ()) == FALSE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
 		                     _("You must be authenticated to upload a video."));
 		return NULL;
 	}
 
-	file_info = g_file_query_info (video_file, "standard::display-name,standard::content-type", G_FILE_QUERY_INFO_NONE, NULL, error);
-	if (file_info == NULL)
-		return NULL;
-
-	slug = g_file_info_get_display_name (file_info);
-	content_type = g_file_info_get_content_type (file_info);
-
 	/* Streaming upload support using GDataUploadStream; automatically handles the XML and multipart stuff for us */
-	output_stream = gdata_upload_stream_new (GDATA_SERVICE (self), SOUP_METHOD_POST,
-	                                         "http://uploads.gdata.youtube.com/feeds/api/users/default/uploads",
-	                                         GDATA_ENTRY (video), slug, content_type);
+	return GDATA_UPLOAD_STREAM (gdata_upload_stream_new (GDATA_SERVICE (self), get_youtube_authorization_domain (), SOUP_METHOD_POST,
+	                                                     "https://uploads.gdata.youtube.com/feeds/api/users/default/uploads",
+	                                                      GDATA_ENTRY (video), slug, content_type, cancellable));
+}
 
-	g_object_unref (file_info);
-	if (output_stream == NULL)
+/**
+ * gdata_youtube_service_finish_video_upload:
+ * @self: a #GDataYouTubeService
+ * @upload_stream: the #GDataUploadStream from the operation
+ * @error: a #GError, or %NULL
+ *
+ * Finish off a video upload operation started by gdata_youtube_service_upload_video(), parsing the result and returning the new #GDataYouTubeVideo.
+ *
+ * If an error occurred during the upload operation, it will have been returned during the operation (e.g. by g_output_stream_splice() or one
+ * of the other stream methods). In such a case, %NULL will be returned but @error will remain unset. @error is only set in the case that the server
+ * indicates that the operation was successful, but an error is encountered in parsing the result sent by the server.
+ *
+ * Return value: (transfer full): the new #GDataYouTubeVideo, or %NULL; unref with g_object_unref()
+ *
+ * Since: 0.8.0
+ */
+GDataYouTubeVideo *
+gdata_youtube_service_finish_video_upload (GDataYouTubeService *self, GDataUploadStream *upload_stream, GError **error)
+{
+	const gchar *response_body;
+	gssize response_length;
+
+	g_return_val_if_fail (GDATA_IS_YOUTUBE_SERVICE (self), NULL);
+	g_return_val_if_fail (GDATA_IS_UPLOAD_STREAM (upload_stream), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	/* Get the response from the server */
+	response_body = gdata_upload_stream_get_response (upload_stream, &response_length);
+	if (response_body == NULL || response_length == 0)
 		return NULL;
 
-	/* Open the video file for reading */
-	input_stream = G_INPUT_STREAM (g_file_read (video_file, cancellable, error));
-	if (input_stream == NULL) {
-		g_object_unref (output_stream);
-		return NULL;
-	}
-
-	/* Splice the streams to upload the file and metadata */
-	g_output_stream_splice (output_stream, input_stream, G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
-	                        cancellable, &child_error);
-
-	g_object_unref (input_stream);
-	if (child_error != NULL) {
-		g_object_unref (output_stream);
-		g_propagate_error (error, child_error);
-		return NULL;
-	}
-
-	/* Get and parse the response from the server */
-	response_body = gdata_upload_stream_get_response (GDATA_UPLOAD_STREAM (output_stream), &response_length);
-	g_assert (response_body != NULL && response_length > 0);
-
-	new_entry = GDATA_YOUTUBE_VIDEO (gdata_parsable_new_from_xml (GDATA_TYPE_YOUTUBE_VIDEO, response_body, (gint) response_length, error));
-	g_object_unref (output_stream);
-
-	return new_entry;
+	/* Parse the response to produce a GDataYouTubeVideo */
+	return GDATA_YOUTUBE_VIDEO (gdata_parsable_new_from_xml (GDATA_TYPE_YOUTUBE_VIDEO, response_body, (gint) response_length, error));
 }
 
 /**
@@ -758,25 +879,10 @@ gdata_youtube_service_get_developer_key (GDataYouTubeService *self)
 	return self->priv->developer_key;
 }
 
-/**
- * gdata_youtube_service_get_youtube_user:
- * @self: a #GDataYouTubeService
- *
- * Gets the #GDataYouTubeService:youtube-user property from the #GDataYouTubeService.
- *
- * Return value: the YouTube username property
- **/
-const gchar *
-gdata_youtube_service_get_youtube_user (GDataYouTubeService *self)
-{
-	g_return_val_if_fail (GDATA_IS_YOUTUBE_SERVICE (self), NULL);
-	return self->priv->youtube_user;
-}
-
 /**
  * gdata_youtube_service_get_categories:
  * @self: a #GDataYouTubeService
- * @cancellable: a #GCancellable, or %NULL
+ * @cancellable: (allow-none): a #GCancellable, or %NULL
  * @error: a #GError, or %NULL
  *
  * Gets a list of the categories currently in use on YouTube. The returned #GDataAPPCategories contains a list of #GDataYouTubeCategory<!-- -->s which
@@ -799,7 +905,8 @@ gdata_youtube_service_get_categories (GDataYouTubeService *self, GCancellable *c
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* Download the category list. Note that this is (service) locale-dependent. */
-	message = _gdata_service_query (GDATA_SERVICE (self), "http://gdata.youtube.com/schemas/2007/categories.cat", NULL, cancellable, error);
+	message = _gdata_service_query (GDATA_SERVICE (self), get_youtube_authorization_domain (),
+	                                "https://gdata.youtube.com/schemas/2007/categories.cat", NULL, cancellable, error);
 	if (message == NULL)
 		return NULL;
 
@@ -831,7 +938,7 @@ get_categories_thread (GSimpleAsyncResult *result, GDataYouTubeService *service,
 /**
  * gdata_youtube_service_get_categories_async:
  * @self: a #GDataYouTubeService
- * @cancellable: optional #GCancellable object, or %NULL
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
  * @callback: a #GAsyncReadyCallback to call when the request is finished
  * @user_data: (closure): data to pass to the @callback function
  *
