@@ -25,7 +25,9 @@
 #include <libsoup/soup.h>
 
 #include <gdata/gdata-service.h>
+#include <gdata/gdata-upload-stream.h>
 #include <gdata/services/youtube/gdata-youtube-video.h>
+#include <gdata/app/gdata-app-categories.h>
 
 G_BEGIN_DECLS
 
@@ -43,7 +45,7 @@ G_BEGIN_DECLS
  * @GDATA_YOUTUBE_RECENTLY_FEATURED_FEED: This feed contains videos recently featured on the YouTube home page or featured videos tab.
  * @GDATA_YOUTUBE_WATCH_ON_MOBILE_FEED: This feed contains videos suitable for playback on mobile devices.
  *
- * Standard feed types for standard feed queries with gata_youtube_service_query_standard_feed(). For more information, see
+ * Standard feed types for standard feed queries with gdata_youtube_service_query_standard_feed(). For more information, see
  * the <ulink type="http" url="http://code.google.com/apis/youtube/2.0/developers_guide_protocol.html#Standard_feeds">online documentation</ulink>.
  **/
 typedef enum {
@@ -105,43 +107,47 @@ typedef struct {
 GType gdata_youtube_service_get_type (void) G_GNUC_CONST;
 GQuark gdata_youtube_service_error_quark (void) G_GNUC_CONST;
 
-GDataYouTubeService *gdata_youtube_service_new (const gchar *developer_key, const gchar *client_id) G_GNUC_WARN_UNUSED_RESULT;
+GDataYouTubeService *gdata_youtube_service_new (const gchar *developer_key, GDataAuthorizer *authorizer) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
+
+GDataAuthorizationDomain *gdata_youtube_service_get_primary_authorization_domain (void) G_GNUC_CONST;
 
 GDataFeed *gdata_youtube_service_query_standard_feed (GDataYouTubeService *self, GDataYouTubeStandardFeedType feed_type, GDataQuery *query,
-						      GCancellable *cancellable,
-						      GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-						      GError **error) G_GNUC_WARN_UNUSED_RESULT;
+                                                      GCancellable *cancellable,
+                                                      GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                                      GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
 void gdata_youtube_service_query_standard_feed_async (GDataYouTubeService *self, GDataYouTubeStandardFeedType feed_type, GDataQuery *query,
-						      GCancellable *cancellable,
-						      GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-						      GAsyncReadyCallback callback, gpointer user_data);
+                                                      GCancellable *cancellable,
+                                                      GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                                      GAsyncReadyCallback callback, gpointer user_data);
 
 GDataFeed *gdata_youtube_service_query_videos (GDataYouTubeService *self, GDataQuery *query,
-					       GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-					       GError **error) G_GNUC_WARN_UNUSED_RESULT;
+                                               GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                               GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
 void gdata_youtube_service_query_videos_async (GDataYouTubeService *self, GDataQuery *query,
-					       GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-					       GAsyncReadyCallback callback, gpointer user_data);
-
-GDataYouTubeVideo *gdata_youtube_service_query_single_video (GDataYouTubeService *self, GDataQuery *query, const gchar *video_id,
-							     GCancellable *cancellable, GError **error) G_GNUC_WARN_UNUSED_RESULT;
-void gdata_youtube_service_query_single_video_async (GDataYouTubeService *self, GDataQuery *query, const gchar *video_id,
-						     GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data);
-GDataYouTubeVideo *gdata_youtube_service_query_single_video_finish (GDataYouTubeService *self, GAsyncResult *async_result,
-								    GError **error);
+                                               GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                               GAsyncReadyCallback callback, gpointer user_data);
 
 GDataFeed *gdata_youtube_service_query_related (GDataYouTubeService *self, GDataYouTubeVideo *video, GDataQuery *query,
-						GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-						GError **error) G_GNUC_WARN_UNUSED_RESULT;
+                                                GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                                GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
 void gdata_youtube_service_query_related_async (GDataYouTubeService *self, GDataYouTubeVideo *video, GDataQuery *query,
-						GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-						GAsyncReadyCallback callback, gpointer user_data);
+                                                GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                                GAsyncReadyCallback callback, gpointer user_data);
 
-GDataYouTubeVideo *gdata_youtube_service_upload_video (GDataYouTubeService *self, GDataYouTubeVideo *video, GFile *video_file,
-						       GCancellable *cancellable, GError **error) G_GNUC_WARN_UNUSED_RESULT;
+GDataUploadStream *gdata_youtube_service_upload_video (GDataYouTubeService *self, GDataYouTubeVideo *video, const gchar *slug,
+                                                       const gchar *content_type, GCancellable *cancellable,
+                                                       GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
+GDataYouTubeVideo *gdata_youtube_service_finish_video_upload (GDataYouTubeService *self, GDataUploadStream *upload_stream,
+                                                              GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
 
-const gchar *gdata_youtube_service_get_developer_key (GDataYouTubeService *self);
-const gchar *gdata_youtube_service_get_youtube_user (GDataYouTubeService *self);
+const gchar *gdata_youtube_service_get_developer_key (GDataYouTubeService *self) G_GNUC_PURE;
+
+GDataAPPCategories *gdata_youtube_service_get_categories (GDataYouTubeService *self, GCancellable *cancellable,
+                                                          GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
+void gdata_youtube_service_get_categories_async (GDataYouTubeService *self, GCancellable *cancellable, GAsyncReadyCallback callback,
+                                                 gpointer user_data);
+GDataAPPCategories *gdata_youtube_service_get_categories_finish (GDataYouTubeService *self, GAsyncResult *async_result,
+                                                                 GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
 
 G_END_DECLS
 
