@@ -76,28 +76,70 @@ static void
 test_authentication (void)
 {
 	gboolean retval;
-	GDataService *service;
+	GDataClientLoginAuthorizer *authorizer;
 	GError *error = NULL;
 
-	/* Create a service */
-	service = GDATA_SERVICE (gdata_contacts_service_new (CLIENT_ID));
+	/* Create an authorizer */
+	authorizer = gdata_client_login_authorizer_new (CLIENT_ID, GDATA_TYPE_CONTACTS_SERVICE);
 
-	g_assert (service != NULL);
-	g_assert (GDATA_IS_SERVICE (service));
-	g_assert_cmpstr (gdata_service_get_client_id (service), ==, CLIENT_ID);
+	g_assert_cmpstr (gdata_client_login_authorizer_get_client_id (authorizer), ==, CLIENT_ID);
 
 	/* Log in */
-	retval = gdata_service_authenticate (service, USERNAME, PASSWORD, NULL, &error);
+	retval = gdata_client_login_authorizer_authenticate (authorizer, USERNAME, PASSWORD, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (retval == TRUE);
 	g_clear_error (&error);
 
 	/* Check all is as it should be */
-	g_assert (gdata_service_is_authenticated (service) == TRUE);
-	g_assert_cmpstr (gdata_service_get_username (service), ==, USERNAME);
-	g_assert_cmpstr (gdata_service_get_password (service), ==, PASSWORD);
+	g_assert_cmpstr (gdata_client_login_authorizer_get_username (authorizer), ==, USERNAME);
+	g_assert_cmpstr (gdata_client_login_authorizer_get_password (authorizer), ==, PASSWORD);
 
-	g_object_unref (service);
+	g_assert (gdata_authorizer_is_authorized_for_domain (GDATA_AUTHORIZER (authorizer),
+	                                                     gdata_contacts_service_get_primary_authorization_domain ()) == TRUE);
+
+	g_object_unref (authorizer);
+}
+
+static void
+test_authentication_async_cb (GDataClientLoginAuthorizer *authorizer, GAsyncResult *async_result, GMainLoop *main_loop)
+{
+	gboolean retval;
+	GError *error = NULL;
+
+	retval = gdata_client_login_authorizer_authenticate_finish (authorizer, async_result, &error);
+	g_assert_no_error (error);
+	g_assert (retval == TRUE);
+	g_clear_error (&error);
+
+	g_main_loop_quit (main_loop);
+
+	/* Check all is as it should be */
+	g_assert_cmpstr (gdata_client_login_authorizer_get_username (authorizer), ==, USERNAME);
+	g_assert_cmpstr (gdata_client_login_authorizer_get_password (authorizer), ==, PASSWORD);
+
+	g_assert (gdata_authorizer_is_authorized_for_domain (GDATA_AUTHORIZER (authorizer),
+	                                                     gdata_contacts_service_get_primary_authorization_domain ()) == TRUE);
+}
+
+static void
+test_authentication_async (void)
+{
+	GMainLoop *main_loop;
+	GDataClientLoginAuthorizer *authorizer;
+
+	/* Create an authorizer */
+	authorizer = gdata_client_login_authorizer_new (CLIENT_ID, GDATA_TYPE_CONTACTS_SERVICE);
+
+	g_assert_cmpstr (gdata_client_login_authorizer_get_client_id (authorizer), ==, CLIENT_ID);
+
+	main_loop = g_main_loop_new (NULL, TRUE);
+	gdata_client_login_authorizer_authenticate_async (authorizer, USERNAME, PASSWORD, NULL,
+	                                                  (GAsyncReadyCallback) test_authentication_async_cb, main_loop);
+
+	g_main_loop_run (main_loop);
+
+	g_main_loop_unref (main_loop);
+	g_object_unref (authorizer);
 }
 
 static void
@@ -162,14 +204,14 @@ test_insert_simple (gconstpointer service)
 	GDataGContactCalendar *calendar;
 	GDataGContactExternalID *external_id;
 	GDataGContactLanguage *language;
-	gchar *xml, *nickname, *billing_information, *directory_server, *gender, *initials, *maiden_name, *mileage, *occupation;
-	gchar *priority, *sensitivity, *short_name, *subject;
+	gchar *nickname, *billing_information, *directory_server, *gender, *initials, *maiden_name, *mileage, *occupation;
+	gchar *priority, *sensitivity, *short_name, *subject, *photo_etag;
 	GList *list;
 	GDate date, *date2;
 	GHashTable *properties;
 	GTimeVal current_time;
 	gint64 edited, creation_time;
-	gboolean deleted, has_photo, birthday_has_year;
+	gboolean deleted, birthday_has_year;
 	GError *error = NULL;
 
 	contact = gdata_contacts_contact_new (NULL);
@@ -276,12 +318,13 @@ test_insert_simple (gconstpointer service)
 	gdata_contacts_contact_set_user_defined_field (contact, "Favourite colour", "Blue");
 	gdata_contacts_contact_set_user_defined_field (contact, "Owes me", "£10");
 	gdata_contacts_contact_set_user_defined_field (contact, "My notes", "");
+	gdata_contacts_contact_set_user_defined_field (contact, "", "Foo"); /* bgo#648058 */
 
 	/* Check the properties of the object */
 	g_object_get (G_OBJECT (contact),
 	              "edited", &edited,
 	              "deleted", &deleted,
-	              "has-photo", &has_photo,
+	              "photo-etag", &photo_etag,
 	              "name", &name,
 	              "nickname", &nickname,
 	              "birthday", &date2,
@@ -301,7 +344,7 @@ test_insert_simple (gconstpointer service)
 
 	g_assert_cmpint (edited, ==, current_time.tv_sec);
 	g_assert (deleted == FALSE);
-	g_assert (has_photo == FALSE);
+	g_assert (photo_etag == NULL);
 	g_assert (name2 == name);
 	g_assert_cmpstr (nickname, ==, "Big J");
 	g_assert (g_date_valid (date2) == TRUE);
@@ -334,10 +377,10 @@ test_insert_simple (gconstpointer service)
 	g_free (sensitivity);
 	g_free (short_name);
 	g_free (subject);
+	g_free (photo_etag);
 
 	/* Check the XML */
-	xml = gdata_parsable_get_xml (GDATA_PARSABLE (contact));
-	g_assert_cmpstr (xml, ==,
+	gdata_test_assert_xml (contact,
 			 "<?xml version='1.0' encoding='UTF-8'?>"
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 				"xmlns:gd='http://schemas.google.com/g/2005' "
@@ -375,6 +418,7 @@ test_insert_simple (gconstpointer service)
 				"<gContact:userDefinedField key='Favourite colour' value='Blue'/>"
 				"<gContact:userDefinedField key='Owes me' value='£10'/>"
 				"<gContact:userDefinedField key='My notes' value=''/>"
+				"<gContact:userDefinedField key='' value='Foo'/>" /* bgo#648058 */
 				"<gContact:hobby>Rowing</gContact:hobby>"
 				"<gContact:nickname>Big J</gContact:nickname>"
 				"<gContact:birthday when='--01-01'/>"
@@ -390,7 +434,6 @@ test_insert_simple (gconstpointer service)
 				"<gContact:shortName>Jon</gContact:shortName>"
 				"<gContact:subject>Charity work</gContact:subject>"
 			 "</entry>");
-	g_free (xml);
 
 	/* Insert the contact */
 	new_contact = gdata_contacts_service_insert_contact (GDATA_CONTACTS_SERVICE (service), contact, NULL, &error);
@@ -512,10 +555,11 @@ test_insert_simple (gconstpointer service)
 	g_assert_cmpstr (gdata_contacts_contact_get_user_defined_field (new_contact, "Favourite colour"), ==, "Blue");
 	g_assert_cmpstr (gdata_contacts_contact_get_user_defined_field (new_contact, "Owes me"), ==, "£10");
 	g_assert_cmpstr (gdata_contacts_contact_get_user_defined_field (new_contact, "My notes"), ==, "");
+	g_assert_cmpstr (gdata_contacts_contact_get_user_defined_field (new_contact, ""), ==, "Foo");
 
 	properties = gdata_contacts_contact_get_user_defined_fields (new_contact);
 	g_assert (properties != NULL);
-	g_assert_cmpuint (g_hash_table_size (properties), ==, 3);
+	g_assert_cmpuint (g_hash_table_size (properties), ==, 4);
 
 	/* Groups */
 	list = gdata_contacts_contact_get_groups (new_contact);
@@ -594,7 +638,9 @@ test_update_simple (gconstpointer service)
 	g_assert (gdata_contacts_contact_set_extended_property (contact, "contact-test", "value"));
 
 	/* Update the contact */
-	new_contact = GDATA_CONTACTS_CONTACT (gdata_service_update_entry (GDATA_SERVICE (service), GDATA_ENTRY (contact), NULL, &error));
+	new_contact = GDATA_CONTACTS_CONTACT (gdata_service_update_entry (GDATA_SERVICE (service),
+	                                                                  gdata_contacts_service_get_primary_authorization_domain (),
+	                                                                  GDATA_ENTRY (contact), NULL, &error));
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_CONTACTS_CONTACT (new_contact));
 	check_kind (GDATA_ENTRY (new_contact), "http://schemas.google.com/contact/2008#contact");
@@ -663,7 +709,7 @@ test_insert_group (gconstpointer service)
 	GHashTable *properties;
 	gint64 edited;
 	gboolean deleted;
-	gchar *system_group_id, *xml;
+	gchar *system_group_id;
 	GError *error = NULL;
 
 	group = gdata_contacts_group_new (NULL);
@@ -701,8 +747,7 @@ test_insert_group (gconstpointer service)
 	g_free (system_group_id);
 
 	/* Check the XML */
-	xml = gdata_parsable_get_xml (GDATA_PARSABLE (group));
-	g_assert_cmpstr (xml, ==,
+	gdata_test_assert_xml (group,
 		"<?xml version='1.0' encoding='UTF-8'?>"
 		"<entry xmlns='http://www.w3.org/2005/Atom' "
 		       "xmlns:gd='http://schemas.google.com/g/2005' "
@@ -713,7 +758,6 @@ test_insert_group (gconstpointer service)
 			"<category term='http://schemas.google.com/contact/2008#group' scheme='http://schemas.google.com/g/2005#kind'/>"
 			"<gd:extendedProperty name='foobar'>barfoo</gd:extendedProperty>"
 		"</entry>");
-	g_free (xml);
 
 	/* Insert the group */
 	new_group = gdata_contacts_service_insert_group (GDATA_CONTACTS_SERVICE (service), group, NULL, &error);
@@ -733,7 +777,8 @@ test_insert_group (gconstpointer service)
 	g_assert_cmpstr (gdata_contacts_group_get_extended_property (new_group, "foobar"), ==, "barfoo");
 
 	/* Delete the group, just to be tidy */
-	g_assert (gdata_service_delete_entry (GDATA_SERVICE (service), GDATA_ENTRY (new_group), NULL, &error) == TRUE);
+	g_assert (gdata_service_delete_entry (GDATA_SERVICE (service), gdata_contacts_service_get_primary_authorization_domain (),
+	                                      GDATA_ENTRY (new_group), NULL, &error) == TRUE);
 	g_assert_no_error (error);
 	g_clear_error (&error);
 
@@ -755,7 +800,8 @@ test_insert_group_async_cb (GDataService *service, GAsyncResult *async_result, G
 	/* TODO: Tests? */
 
 	/* Delete the group, just to be tidy */
-	g_assert (gdata_service_delete_entry (GDATA_SERVICE (service), entry, NULL, &error) == TRUE);
+	g_assert (gdata_service_delete_entry (GDATA_SERVICE (service), gdata_contacts_service_get_primary_authorization_domain (),
+	                                      entry, NULL, &error) == TRUE);
 	g_assert_no_error (error);
 	g_clear_error (&error);
 
@@ -789,6 +835,81 @@ test_insert_group_async (gconstpointer service)
 	g_object_unref (group);
 }
 
+static void
+test_contact_escaping (void)
+{
+	GDataContactsContact *contact;
+
+	contact = gdata_contacts_contact_new (NULL);
+	gdata_contacts_contact_set_nickname (contact, "Nickname & stuff");
+	gdata_contacts_contact_set_billing_information (contact, "Billing information & stuff");
+	gdata_contacts_contact_set_directory_server (contact, "http://foo.com?foo&bar");
+	gdata_contacts_contact_set_gender (contact, "Misc. & other");
+	gdata_contacts_contact_set_initials (contact, "<AB>");
+	gdata_contacts_contact_set_maiden_name (contact, "Maiden & name");
+	gdata_contacts_contact_set_mileage (contact, "Over the hills & far away");
+	gdata_contacts_contact_set_occupation (contact, "Occupation & stuff");
+	gdata_contacts_contact_set_priority (contact, "http://foo.com?foo&priority=bar");
+	gdata_contacts_contact_set_sensitivity (contact, "http://foo.com?foo&sensitivity=bar");
+	gdata_contacts_contact_set_short_name (contact, "Short name & stuff");
+	gdata_contacts_contact_set_subject (contact, "Subject & stuff");
+	gdata_contacts_contact_add_hobby (contact, "Escaping &s");
+	gdata_contacts_contact_set_extended_property (contact, "extended & prop", "<unescaped>Value should be a pre-escaped XML blob.</unescaped>");
+	gdata_contacts_contact_set_user_defined_field (contact, "User defined field & stuff", "Value & stuff");
+	gdata_contacts_contact_add_group (contact, "http://foo.com?foo&bar");
+
+	/* Check the outputted XML is escaped properly */
+	gdata_test_assert_xml (contact,
+	                 "<?xml version='1.0' encoding='UTF-8'?>"
+	                 "<entry xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' "
+	                        "xmlns:app='http://www.w3.org/2007/app' xmlns:gContact='http://schemas.google.com/contact/2008'>"
+				"<title type='text'></title>"
+				"<category term='http://schemas.google.com/contact/2008#contact' scheme='http://schemas.google.com/g/2005#kind'/>"
+				"<gd:name/>"
+				"<gd:extendedProperty name='extended &amp; prop'>"
+					"<unescaped>Value should be a pre-escaped XML blob.</unescaped>"
+				"</gd:extendedProperty>"
+				"<gContact:userDefinedField key='User defined field &amp; stuff' value='Value &amp; stuff'/>"
+				"<gContact:groupMembershipInfo href='http://foo.com?foo&amp;bar'/>"
+				"<gContact:hobby>Escaping &amp;s</gContact:hobby>"
+				"<gContact:nickname>Nickname &amp; stuff</gContact:nickname>"
+				"<gContact:billingInformation>Billing information &amp; stuff</gContact:billingInformation>"
+				"<gContact:directoryServer>http://foo.com?foo&amp;bar</gContact:directoryServer>"
+				"<gContact:gender value='Misc. &amp; other'/>"
+				"<gContact:initials>&lt;AB&gt;</gContact:initials>"
+				"<gContact:maidenName>Maiden &amp; name</gContact:maidenName>"
+				"<gContact:mileage>Over the hills &amp; far away</gContact:mileage>"
+				"<gContact:occupation>Occupation &amp; stuff</gContact:occupation>"
+				"<gContact:priority rel='http://foo.com?foo&amp;priority=bar'/>"
+				"<gContact:sensitivity rel='http://foo.com?foo&amp;sensitivity=bar'/>"
+				"<gContact:shortName>Short name &amp; stuff</gContact:shortName>"
+				"<gContact:subject>Subject &amp; stuff</gContact:subject>"
+	                 "</entry>");
+	g_object_unref (contact);
+}
+
+static void
+test_group_escaping (void)
+{
+	GDataContactsGroup *group;
+
+	group = gdata_contacts_group_new (NULL);
+	gdata_contacts_group_set_extended_property (group, "extended & prop", "<unescaped>Value should be a pre-escaped XML blob.</unescaped>");
+
+	/* Check the outputted XML is escaped properly */
+	gdata_test_assert_xml (group,
+	                 "<?xml version='1.0' encoding='UTF-8'?>"
+	                 "<entry xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' "
+	                        "xmlns:app='http://www.w3.org/2007/app' xmlns:gContact='http://schemas.google.com/contact/2008'>"
+				"<title type='text'></title>"
+				"<category term='http://schemas.google.com/contact/2008#group' scheme='http://schemas.google.com/g/2005#kind'/>"
+				"<gd:extendedProperty name='extended &amp; prop'>"
+					"<unescaped>Value should be a pre-escaped XML blob.</unescaped>"
+				"</gd:extendedProperty>"
+	                 "</entry>");
+	g_object_unref (group);
+}
+
 static void
 test_query_uri (void)
 {
@@ -1227,7 +1348,6 @@ test_parser_error_handling (void)
 	TEST_XML_ERROR_HANDLING ("<gContact:userDefinedField/>"); /* no key or value */
 	TEST_XML_ERROR_HANDLING ("<gContact:userDefinedField key='foo'/>"); /* no value */
 	TEST_XML_ERROR_HANDLING ("<gContact:userDefinedField value='bar'/>"); /* no key */
-	TEST_XML_ERROR_HANDLING ("<gContact:userDefinedField key='' value='bar'/>"); /* empty key */
 
 	/* gContact:groupMembershipInfo */
 	TEST_XML_ERROR_HANDLING ("<gContact:groupMembershipInfo/>");
@@ -1453,7 +1573,7 @@ test_photo_has_photo (gconstpointer service)
 	g_clear_error (&error);
 
 	/* Check for no photo */
-	g_assert (gdata_contacts_contact_has_photo (contact) == FALSE);
+	g_assert (gdata_contacts_contact_get_photo_etag (contact) == NULL);
 	g_assert (gdata_contacts_contact_get_photo (contact, GDATA_CONTACTS_SERVICE (service), &length, &content_type, NULL, &error) == NULL);
 	g_assert_cmpint (length, ==, 0);
 	g_assert (content_type == NULL);
@@ -1480,7 +1600,7 @@ test_photo_has_photo (gconstpointer service)
 	check_kind (GDATA_ENTRY (contact), "http://schemas.google.com/contact/2008#contact");
 	g_clear_error (&error);
 
-	g_assert (gdata_contacts_contact_has_photo (contact) == TRUE);
+	g_assert (gdata_contacts_contact_get_photo_etag (contact) != NULL);
 	g_object_unref (contact);
 }
 
@@ -1488,17 +1608,17 @@ static void
 test_photo_add (gconstpointer service)
 {
 	GDataContactsContact *contact;
-	gchar *data;
+	guint8 *data;
 	gsize length;
 	gboolean retval;
 	GError *error = NULL;
 
 	/* Get the photo */
-	g_assert (g_file_get_contents (TEST_FILE_DIR "photo.jpg", &data, &length, NULL) == TRUE);
+	g_assert (g_file_get_contents (TEST_FILE_DIR "photo.jpg", (gchar**) &data, &length, NULL) == TRUE);
 
 	/* Add it to the contact */
 	contact = get_contact (service);
-	retval = gdata_contacts_contact_set_photo (contact, GDATA_SERVICE (service), data, length, NULL, &error);
+	retval = gdata_contacts_contact_set_photo (contact, GDATA_CONTACTS_SERVICE (service), data, length, "image/jpeg", NULL, &error);
 	g_assert_no_error (error);
 	g_assert (retval == TRUE);
 
@@ -1507,16 +1627,56 @@ test_photo_add (gconstpointer service)
 	g_free (data);
 }
 
+static void
+test_photo_add_async_cb (GDataContactsContact *contact, GAsyncResult *result, GMainLoop *main_loop)
+{
+	gboolean success;
+	GError *error = NULL;
+
+	success = gdata_contacts_contact_set_photo_finish (contact, result, &error);
+	g_assert_no_error (error);
+	g_assert (success == TRUE);
+
+	g_assert (gdata_contacts_contact_get_photo_etag (contact) != NULL);
+
+	g_main_loop_quit (main_loop);
+}
+
+static void
+test_photo_add_async (gconstpointer service)
+{
+	GDataContactsContact *contact;
+	guint8 *data;
+	gsize length;
+	GMainLoop *main_loop;
+
+	/* Get the photo */
+	g_assert (g_file_get_contents (TEST_FILE_DIR "photo.jpg", (gchar**) &data, &length, NULL) == TRUE);
+
+	contact = get_contact (service);
+	main_loop = g_main_loop_new (NULL, TRUE);
+
+	/* Add it to the contact asynchronously */
+	gdata_contacts_contact_set_photo_async (contact, GDATA_CONTACTS_SERVICE (service), data, length, "image/jpeg", NULL,
+	                                        (GAsyncReadyCallback) test_photo_add_async_cb, main_loop);
+	g_main_loop_run (main_loop);
+
+	g_main_loop_unref (main_loop);
+	g_object_unref (contact);
+	g_free (data);
+}
+
 static void
 test_photo_get (gconstpointer service)
 {
 	GDataContactsContact *contact;
-	gchar *data, *content_type = NULL;
+	guint8 *data;
+	gchar *content_type = NULL;
 	gsize length = 0;
 	GError *error = NULL;
 
 	contact = get_contact (service);
-	g_assert (gdata_contacts_contact_has_photo (contact) == TRUE);
+	g_assert (gdata_contacts_contact_get_photo_etag (contact) != NULL);
 
 	/* Get the photo from the network */
 	data = gdata_contacts_contact_get_photo (contact, GDATA_CONTACTS_SERVICE (service), &length, &content_type, NULL, &error);
@@ -1525,7 +1685,7 @@ test_photo_get (gconstpointer service)
 	g_assert (length != 0);
 	g_assert_cmpstr (content_type, ==, "image/jpeg");
 
-	g_assert (gdata_contacts_contact_has_photo (contact) == TRUE);
+	g_assert (gdata_contacts_contact_get_photo_etag (contact) != NULL);
 
 	g_free (content_type);
 	g_free (data);
@@ -1533,6 +1693,49 @@ test_photo_get (gconstpointer service)
 	g_clear_error (&error);
 }
 
+static void
+test_photo_get_async_cb (GDataContactsContact *contact, GAsyncResult *result, GMainLoop *main_loop)
+{
+	guint8 *data;
+	gsize length;
+	gchar *content_type;
+	GError *error = NULL;
+
+	/* Finish getting the photo */
+	data = gdata_contacts_contact_get_photo_finish (contact, result, &length, &content_type, &error);
+	g_assert_no_error (error);
+	g_assert (data != NULL);
+	g_assert (length != 0);
+	g_assert_cmpstr (content_type, ==, "image/jpeg");
+
+	g_assert (gdata_contacts_contact_get_photo_etag (contact) != NULL);
+
+	g_main_loop_quit (main_loop);
+
+	g_free (content_type);
+	g_free (data);
+}
+
+static void
+test_photo_get_async (gconstpointer service)
+{
+	GDataContactsContact *contact;
+	GMainLoop *main_loop;
+
+	contact = get_contact (service);
+	g_assert (gdata_contacts_contact_get_photo_etag (contact) != NULL);
+
+	main_loop = g_main_loop_new (NULL, TRUE);
+
+	/* Get the photo from the network asynchronously */
+	gdata_contacts_contact_get_photo_async (contact, GDATA_CONTACTS_SERVICE (service), NULL, (GAsyncReadyCallback) test_photo_get_async_cb,
+	                                        main_loop);
+	g_main_loop_run (main_loop);
+
+	g_main_loop_unref (main_loop);
+	g_object_unref (contact);
+}
+
 static void
 test_photo_delete (gconstpointer service)
 {
@@ -1540,18 +1743,53 @@ test_photo_delete (gconstpointer service)
 	GError *error = NULL;
 
 	contact = get_contact (service);
-	g_assert (gdata_contacts_contact_has_photo (contact) == TRUE);
+	g_assert (gdata_contacts_contact_get_photo_etag (contact) != NULL);
 
 	/* Remove the contact's photo */
-	g_assert (gdata_contacts_contact_set_photo (contact, GDATA_SERVICE (service), NULL, 0, NULL, &error) == TRUE);
+	g_assert (gdata_contacts_contact_set_photo (contact, GDATA_CONTACTS_SERVICE (service), NULL, 0, NULL, NULL, &error) == TRUE);
 	g_assert_no_error (error);
 
-	g_assert (gdata_contacts_contact_has_photo (contact) == FALSE);
+	g_assert (gdata_contacts_contact_get_photo_etag (contact) == NULL);
 
 	g_clear_error (&error);
 	g_object_unref (contact);
 }
 
+static void
+test_photo_delete_async_cb (GDataContactsContact *contact, GAsyncResult *result, GMainLoop *main_loop)
+{
+	gboolean success;
+	GError *error = NULL;
+
+	success = gdata_contacts_contact_set_photo_finish (contact, result, &error);
+	g_assert_no_error (error);
+	g_assert (success == TRUE);
+
+	g_assert (gdata_contacts_contact_get_photo_etag (contact) == NULL);
+
+	g_main_loop_quit (main_loop);
+}
+
+static void
+test_photo_delete_async (gconstpointer service)
+{
+	GDataContactsContact *contact;
+	GMainLoop *main_loop;
+
+	contact = get_contact (service);
+	main_loop = g_main_loop_new (NULL, TRUE);
+
+	g_assert (gdata_contacts_contact_get_photo_etag (contact) != NULL);
+
+	/* Delete it from the contact asynchronously */
+	gdata_contacts_contact_set_photo_async (contact, GDATA_CONTACTS_SERVICE (service), NULL, 0, NULL, NULL,
+	                                        (GAsyncReadyCallback) test_photo_delete_async_cb, main_loop);
+	g_main_loop_run (main_loop);
+
+	g_main_loop_unref (main_loop);
+	g_object_unref (contact);
+}
+
 static void
 test_batch (gconstpointer service)
 {
@@ -1564,11 +1802,12 @@ test_batch (gconstpointer service)
 	GError *error = NULL, *entry_error = NULL;
 
 	/* Here we hardcode the feed URI, but it should really be extracted from a contacts feed, as the GDATA_LINK_BATCH link */
-	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "http://www.google.com/m8/feeds/contacts/default/full/batch");
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), gdata_contacts_service_get_primary_authorization_domain (),
+	                                              "https://www.google.com/m8/feeds/contacts/default/full/batch");
 
 	/* Check the properties of the operation */
 	g_assert (gdata_batch_operation_get_service (operation) == service);
-	g_assert_cmpstr (gdata_batch_operation_get_feed_uri (operation), ==, "http://www.google.com/m8/feeds/contacts/default/full/batch");
+	g_assert_cmpstr (gdata_batch_operation_get_feed_uri (operation), ==, "https://www.google.com/m8/feeds/contacts/default/full/batch");
 
 	g_object_get (operation,
 	              "service", &service2,
@@ -1576,7 +1815,7 @@ test_batch (gconstpointer service)
 	              NULL);
 
 	g_assert (service2 == service);
-	g_assert_cmpstr (feed_uri, ==, "http://www.google.com/m8/feeds/contacts/default/full/batch");
+	g_assert_cmpstr (feed_uri, ==, "https://www.google.com/m8/feeds/contacts/default/full/batch");
 
 	g_object_unref (service2);
 	g_free (feed_uri);
@@ -1586,7 +1825,7 @@ test_batch (gconstpointer service)
 	gdata_entry_set_title (GDATA_ENTRY (contact), "Fooish Bar");
 
 	gdata_test_batch_operation_insertion (operation, GDATA_ENTRY (contact), &inserted_entry, NULL);
-	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert (gdata_test_batch_operation_run (operation, NULL, &error) == TRUE);
 	g_assert_no_error (error);
 
 	g_clear_error (&error);
@@ -1597,13 +1836,14 @@ test_batch (gconstpointer service)
 	contact2 = gdata_contacts_contact_new (NULL);
 	gdata_entry_set_title (GDATA_ENTRY (contact2), "Brian");
 
-	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "http://www.google.com/m8/feeds/contacts/default/full/batch");
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), gdata_contacts_service_get_primary_authorization_domain (),
+	                                              "https://www.google.com/m8/feeds/contacts/default/full/batch");
 	op_id = gdata_test_batch_operation_insertion (operation, GDATA_ENTRY (contact2), &inserted_entry2, NULL);
 	op_id2 = gdata_test_batch_operation_query (operation, gdata_entry_get_id (inserted_entry), GDATA_TYPE_CONTACTS_CONTACT, inserted_entry, NULL,
 	                                           NULL);
 	g_assert_cmpuint (op_id, !=, op_id2);
 
-	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert (gdata_test_batch_operation_run (operation, NULL, &error) == TRUE);
 	g_assert_no_error (error);
 
 	g_clear_error (&error);
@@ -1614,7 +1854,8 @@ test_batch (gconstpointer service)
 	gdata_entry_set_title (inserted_entry2, "Toby");
 	contact3 = gdata_contacts_contact_new ("foobar");
 
-	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "http://www.google.com/m8/feeds/contacts/default/full/batch");
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), gdata_contacts_service_get_primary_authorization_domain (),
+	                                              "https://www.google.com/m8/feeds/contacts/default/full/batch");
 	op_id = gdata_test_batch_operation_deletion (operation, inserted_entry, NULL);
 	op_id2 = gdata_test_batch_operation_deletion (operation, GDATA_ENTRY (contact3), &entry_error);
 	op_id3 = gdata_test_batch_operation_update (operation, inserted_entry2, &inserted_entry3, NULL);
@@ -1622,7 +1863,7 @@ test_batch (gconstpointer service)
 	g_assert_cmpuint (op_id, !=, op_id3);
 	g_assert_cmpuint (op_id2, !=, op_id3);
 
-	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert (gdata_test_batch_operation_run (operation, NULL, &error) == TRUE);
 	g_assert_no_error (error);
 
 	g_assert_error (entry_error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR);
@@ -1635,9 +1876,10 @@ test_batch (gconstpointer service)
 
 	/* Run another batch operation to update the second entry with the wrong ETag (i.e. pass the old version of the entry to the batch operation
 	 * to test error handling */
-	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "http://www.google.com/m8/feeds/contacts/default/full/batch");
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), gdata_contacts_service_get_primary_authorization_domain (),
+	                                              "https://www.google.com/m8/feeds/contacts/default/full/batch");
 	gdata_test_batch_operation_update (operation, inserted_entry2, NULL, &entry_error);
-	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert (gdata_test_batch_operation_run (operation, NULL, &error) == TRUE);
 	g_assert_no_error (error);
 
 	g_assert_error (entry_error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_CONFLICT);
@@ -1648,9 +1890,10 @@ test_batch (gconstpointer service)
 	g_object_unref (inserted_entry2);
 
 	/* Run a final batch operation to delete the second entry */
-	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "http://www.google.com/m8/feeds/contacts/default/full/batch");
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), gdata_contacts_service_get_primary_authorization_domain (),
+	                                              "https://www.google.com/m8/feeds/contacts/default/full/batch");
 	gdata_test_batch_operation_deletion (operation, inserted_entry3, NULL);
-	g_assert (gdata_batch_operation_run (operation, NULL, &error) == TRUE);
+	g_assert (gdata_test_batch_operation_run (operation, NULL, &error) == TRUE);
 	g_assert_no_error (error);
 
 	g_clear_error (&error);
@@ -1685,7 +1928,10 @@ test_batch_async_cb (GDataBatchOperation *operation, GAsyncResult *async_result,
 {
 	GError *error = NULL;
 
-	g_assert (gdata_batch_operation_run_finish (operation, async_result, &error) == TRUE);
+	/* Clear all pending events (such as callbacks for the operations) */
+	while (g_main_context_iteration (NULL, FALSE) == TRUE);
+
+	g_assert (gdata_test_batch_operation_run_finish (operation, async_result, &error) == TRUE);
 	g_assert_no_error (error);
 	g_clear_error (&error);
 
@@ -1696,13 +1942,13 @@ static void
 test_batch_async (BatchAsyncData *data, gconstpointer service)
 {
 	GDataBatchOperation *operation;
-	guint op_id;
 	GMainLoop *main_loop;
 
 	/* Run an async query operation on the contact */
-	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "http://www.google.com/m8/feeds/contacts/default/full/batch");
-	op_id = gdata_test_batch_operation_query (operation, gdata_entry_get_id (GDATA_ENTRY (data->new_contact)), GDATA_TYPE_CONTACTS_CONTACT,
-	                                          GDATA_ENTRY (data->new_contact), NULL, NULL);
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), gdata_contacts_service_get_primary_authorization_domain (),
+	                                              "https://www.google.com/m8/feeds/contacts/default/full/batch");
+	gdata_test_batch_operation_query (operation, gdata_entry_get_id (GDATA_ENTRY (data->new_contact)), GDATA_TYPE_CONTACTS_CONTACT,
+	                                  GDATA_ENTRY (data->new_contact), NULL, NULL);
 
 	main_loop = g_main_loop_new (NULL, TRUE);
 
@@ -1718,7 +1964,10 @@ test_batch_async_cancellation_cb (GDataBatchOperation *operation, GAsyncResult *
 {
 	GError *error = NULL;
 
-	g_assert (gdata_batch_operation_run_finish (operation, async_result, &error) == FALSE);
+	/* Clear all pending events (such as callbacks for the operations) */
+	while (g_main_context_iteration (NULL, FALSE) == TRUE);
+
+	g_assert (gdata_test_batch_operation_run_finish (operation, async_result, &error) == FALSE);
 	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_CANCELLED);
 	g_clear_error (&error);
 
@@ -1729,14 +1978,15 @@ static void
 test_batch_async_cancellation (BatchAsyncData *data, gconstpointer service)
 {
 	GDataBatchOperation *operation;
-	guint op_id;
 	GMainLoop *main_loop;
 	GCancellable *cancellable;
+	GError *error = NULL;
 
 	/* Run an async query operation on the contact */
-	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), "http://www.google.com/m8/feeds/contacts/default/full/batch");
-	op_id = gdata_test_batch_operation_query (operation, gdata_entry_get_id (GDATA_ENTRY (data->new_contact)), GDATA_TYPE_CONTACTS_CONTACT,
-	                                          GDATA_ENTRY (data->new_contact), NULL, NULL);
+	operation = gdata_batchable_create_operation (GDATA_BATCHABLE (service), gdata_contacts_service_get_primary_authorization_domain (),
+	                                              "https://www.google.com/m8/feeds/contacts/default/full/batch");
+	gdata_test_batch_operation_query (operation, gdata_entry_get_id (GDATA_ENTRY (data->new_contact)), GDATA_TYPE_CONTACTS_CONTACT,
+	                                  GDATA_ENTRY (data->new_contact), NULL, &error);
 
 	main_loop = g_main_loop_new (NULL, TRUE);
 	cancellable = g_cancellable_new ();
@@ -1745,6 +1995,10 @@ test_batch_async_cancellation (BatchAsyncData *data, gconstpointer service)
 	g_cancellable_cancel (cancellable); /* this should cancel the operation before it even starts, as we haven't run the main loop yet */
 
 	g_main_loop_run (main_loop);
+
+	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_CANCELLED);
+	g_clear_error (&error);
+
 	g_main_loop_unref (main_loop);
 	g_object_unref (cancellable);
 	g_object_unref (operation);
@@ -1756,13 +2010,59 @@ teardown_batch_async (BatchAsyncData *data, gconstpointer service)
 	GError *error = NULL;
 
 	/* Delete the contact */
-	g_assert (gdata_service_delete_entry (GDATA_SERVICE (service), GDATA_ENTRY (data->new_contact), NULL, &error) == TRUE);
+	g_assert (gdata_service_delete_entry (GDATA_SERVICE (service), gdata_contacts_service_get_primary_authorization_domain (),
+	                                      GDATA_ENTRY (data->new_contact), NULL, &error) == TRUE);
 	g_assert_no_error (error);
 	g_clear_error (&error);
 
 	g_object_unref (data->new_contact);
 }
 
+static void
+test_groups_membership (void)
+{
+	GDataContactsContact *contact;
+	GList *groups;
+
+	/* Create a new contact with no groups */
+	contact = gdata_contacts_contact_new (NULL);
+	g_assert (GDATA_IS_CONTACTS_CONTACT (contact));
+	g_assert (gdata_contacts_contact_get_groups (contact) == NULL);
+	g_assert (gdata_contacts_contact_is_group_deleted (contact, "http://notagroup.com/") == FALSE);
+
+	/* Add a group */
+	gdata_contacts_contact_add_group (contact, "http://foo.com/group1");
+	g_assert (gdata_contacts_contact_is_group_deleted (contact, "http://foo.com/group1") == FALSE);
+
+	groups = gdata_contacts_contact_get_groups (contact);
+	g_assert_cmpint (g_list_length (groups), ==, 1);
+	g_assert_cmpstr (groups->data, ==, "http://foo.com/group1");
+
+	/* Add another group */
+	gdata_contacts_contact_add_group (contact, "http://foo.com/group2");
+	g_assert (gdata_contacts_contact_is_group_deleted (contact, "http://foo.com/group1") == FALSE);
+	g_assert (gdata_contacts_contact_is_group_deleted (contact, "http://foo.com/group2") == FALSE);
+
+	groups = gdata_contacts_contact_get_groups (contact);
+	g_assert_cmpint (g_list_length (groups), ==, 2);
+	if (strcmp (groups->data, "http://foo.com/group1") == 0)
+		g_assert_cmpstr (groups->next->data, ==, "http://foo.com/group2");
+	else if (strcmp (groups->data, "http://foo.com/group2") == 0)
+		g_assert_cmpstr (groups->next->data, ==, "http://foo.com/group1");
+	else
+		g_assert_not_reached ();
+
+	/* Remove the first group */
+	gdata_contacts_contact_remove_group (contact, "http://foo.com/group1");
+	g_assert (gdata_contacts_contact_is_group_deleted (contact, "http://foo.com/group1") == FALSE); /* hasn't been propagated to the server */
+
+	groups = gdata_contacts_contact_get_groups (contact);
+	g_assert_cmpint (g_list_length (groups), ==, 1);
+	g_assert_cmpstr (groups->data, ==, "http://foo.com/group2");
+
+	g_object_unref (contact);
+}
+
 static void
 test_id (void)
 {
@@ -1798,15 +2098,19 @@ int
 main (int argc, char *argv[])
 {
 	gint retval;
+	GDataAuthorizer *authorizer = NULL;
 	GDataService *service = NULL;
 
 	gdata_test_init (argc, argv);
 
 	if (gdata_test_internet () == TRUE) {
-		service = GDATA_SERVICE (gdata_contacts_service_new (CLIENT_ID));
-		gdata_service_authenticate (service, USERNAME, PASSWORD, NULL, NULL);
+		authorizer = GDATA_AUTHORIZER (gdata_client_login_authorizer_new (CLIENT_ID, GDATA_TYPE_CONTACTS_SERVICE));
+		gdata_client_login_authorizer_authenticate (GDATA_CLIENT_LOGIN_AUTHORIZER (authorizer), USERNAME, PASSWORD, NULL, NULL);
+
+		service = GDATA_SERVICE (gdata_contacts_service_new (authorizer));
 
 		g_test_add_func ("/contacts/authentication", test_authentication);
+		g_test_add_func ("/contacts/authentication_async", test_authentication_async);
 
 		g_test_add_data_func ("/contacts/insert/simple", service, test_insert_simple);
 		g_test_add_data_func ("/contacts/update/simple", service, test_update_simple);
@@ -1817,7 +2121,10 @@ main (int argc, char *argv[])
 		g_test_add_data_func ("/contacts/photo/has_photo", service, test_photo_has_photo);
 		g_test_add_data_func ("/contacts/photo/add", service, test_photo_add);
 		g_test_add_data_func ("/contacts/photo/get", service, test_photo_get);
+		g_test_add_data_func ("/contacts/photo/get/async", service, test_photo_get_async);
 		g_test_add_data_func ("/contacts/photo/delete", service, test_photo_delete);
+		g_test_add_data_func ("/contacts/photo/add/async", service, test_photo_add_async);
+		g_test_add_data_func ("/contacts/photo/delete/async", service, test_photo_delete_async);
 
 		g_test_add_data_func ("/contacts/batch", service, test_batch);
 		g_test_add ("/contacts/batch/async", BatchAsyncData, service, setup_batch_async, test_batch_async, teardown_batch_async);
@@ -1830,6 +2137,9 @@ main (int argc, char *argv[])
 		g_test_add_data_func ("/contacts/groups/insert_async", service, test_insert_group_async);
 	}
 
+	g_test_add_func ("/contacts/contact/escaping", test_contact_escaping);
+	g_test_add_func ("/contacts/group/escaping", test_group_escaping);
+
 	g_test_add_func ("/contacts/query/uri", test_query_uri);
 	g_test_add_func ("/contacts/query/etag", test_query_etag);
 	g_test_add_func ("/contacts/query/properties", test_query_properties);
@@ -1841,6 +2151,7 @@ main (int argc, char *argv[])
 	g_test_add_func ("/contacts/groups/parser/normal", test_groups_parser_normal);
 	g_test_add_func ("/contacts/groups/parser/system", test_groups_parser_system);
 	g_test_add_func ("/contacts/groups/parser/error_handling", test_groups_parser_error_handling);
+	g_test_add_func ("/contacts/groups/membership", test_groups_membership);
 
 	g_test_add_func ("/contacts/id", test_id);
 
